
#include "vmic_common_test.c"


/*
 * Prints, and optionally sets the debug flags
 *
 * USAGE:
 * vdbflags [-s flags]
 *
 * flags - debug flags
 *
 * RETURNS:
 *   0 on success or -1 on error.
 */
int main( int argc, char **argv )
{
  int            c, fd, set_flags;
  UVI_UINT32     flags;

  set_flags = 0;

  while( ( c = getopt( argc, argv, "s:" ) ) != -1 )
    switch( c )
      {
      case 's' :
	if( ( flags = strtol( optarg, NULL, 0 ) ) < 0 )
	  {
	    fprintf( stderr, "Invalid flags\n" );
	    return( -1 );
	  }
	set_flags = 1;
	break;
      case '?' :
      default :
	fprintf( stderr, "vdbflags [-s flags]\n" );
	return( -1 );
      }

  if( ( fd = VmeOpen( ) ) < 0 )
    {
      perror( "VmeOpen" );
      return( -1 );
    }

  if( set_flags )
    if( VmeSetDebugFlags( fd, flags ) != UVI_SUCCESS )
      {
	perror( "VmeSetDebugFlags" );
	VmeClose( fd );
	return( -1 );
      }

  if( VmeGetDebugFlags( fd, &flags ) != UVI_SUCCESS )
    {
      perror( "VmeGetDebugFlags" );
      VmeClose( fd );
      return( -1 );
    }

  fprintf( stdout, "Debug flags: %x\n", flags );

  if( VmeClose( fd )!= UVI_SUCCESS )
    {
      perror( "VmeClose" );
      return( -1 );
    }

  return( 0 );
}
