
#include "vmic_common_test.c"


/*
 * Handler for Ctrl-C
 */
static void ctlc_handler( int v );


/*
 * These variables must be global to be accessed by the Ctrl-C handler
 */
int fd, mb;


/*
 * Set up an interrupt handler to catch mailbox interrupts.
 *
 * USAGE:
 * vmb mailbox
 *
 * mailbox - Mailbox number (0-3).
 *
 *
 * RETURNS:
 *   0 on success or -1 on error.
 */
int main( int argc, char **argv )
{
  UVI_UINT32        mb_ret;
  struct sigaction  act;

  if( argc < 2 )
    {
      fprintf( stderr, "vmb mailbox\n" );
      return( -1 );
    }

  if( ( mb = strtol( argv[1], NULL, 0 ) ) < 0 )
    {
      fprintf( stderr, "Invalid mailbox argument\n" );
      return( -1 );
    }

  /*
   * We do not want to mask any interrupts while handling this signal
   */
  sigemptyset( &act.sa_mask );
  act.sa_handler = ctlc_handler;
  act.sa_flags = 0;
  
  /*
   * Set the interrupt handler for SIGKILL
   */
  if( sigaction( SIGINT, &act, NULL ) < 0 )
    {
      perror( "sigaction" );
      return( -1 );
    }

  if( ( fd = VmeOpen( ) ) < 0 )
    {
      perror( "VmeOpen" );
      return( -1 );
    }

  if( RequestVmeMB( fd, mb ) != UVI_SUCCESS )
    {
      perror( "RequestVmeMB" );
      VmeClose( fd );
      return( -1 );
    }

  if( WaitVmeMBInt( fd, mb, &mb_ret ) != UVI_SUCCESS )
    {
      perror( "WaitVmeMBInt" );
      ReleaseVmeMB( fd, mb );
      VmeClose( fd );
      return( -1 );
    }

  fprintf( stdout, "%x\n", mb_ret );

  if( ReleaseVmeMB( fd, mb ) != UVI_SUCCESS )
    {
      perror( "ReleaseVmeMB" );
      VmeClose( fd );
      return( -1 );
    }

  if( VmeClose( fd )!= UVI_SUCCESS )
    {
      perror( "VmeClose" );
      return( -1 );
    }

  return( 0 );
}


static void ctlc_handler( int v )
{

  DPRINTF(( stderr, "Termination signal received\n" ));
  ReleaseVmeMB( fd, mb );
  VmeClose( fd );
  exit( -1 );
}
