
#include "vmic_common_test.c"


/*
 * Check to see if a VMEbus address is valid
 *
 * USAGE:
 * vprobe [-a address modifier] [-d data width] address
 *
 * address modifier - VMEbus address modifier [A16UP, A16UD, A16SP, A16SD,
 *                                             A24UP, A24UD, A24SP, A24SD,
 *                                             A32UP, A32UD, A32SP, A32SD ]
 *
 * data width - VMEbus access data width [8, 16, 32]
 *
 * address - VMEbus address
 *
 * RETURNS:
 *   0 on success or -1 on error.
 */
int main( int argc, char **argv )
{
  int            c, fd, status;
  UVIHANDLE      handle;
  UVI_UINT32     vaddr, addr_mod, dwidth;

  addr_mod = DEFAULT_ADDR_MOD;
  dwidth = DEFAULT_DWIDTH;

  while( ( c = getopt( argc, argv, "a:d:" ) ) != -1 )
    switch( c )
      {
      case 'a' :
	if( ( addr_mod = strtoam( optarg ) ) < 0 )
	  {
	    fprintf( stderr, "Invalid address modifier\n" );
	    return( -1 );
	  }
	break;
      case 'd' :
	if( ( dwidth = strtodw( optarg ) ) < 0 )
	  {
	    fprintf( stderr, "Invalid data width\n" );
	    return( -1 );
	  }
	break;
      case '?' :
      default :
	fprintf( stderr, "vprobe [-a address modifier] [-d data width] "
		 "address\n" );
	return( -1 );
      }

  if( argv[optind] )
    vaddr = strtoul( argv[optind++], NULL, 0 );
  else
    {
      fprintf( stderr, "No address specified\n" );
      return( -1 );
    }

  DPRINTF(( stdout, "Probing address 0x%x, modifier 0x%x, "
	    "data width %d bits\n", vaddr, addr_mod, dwidth ));

  if( ( fd = VmeOpen( ) ) < 0 )
    {
      perror( "VmeOpen" );
      return( -1 );
    }

  status = VmeProbeAddress( fd, vaddr, addr_mod, dwidth );

  switch( status )
    {
    case UVI_SUCCESS :
      fprintf( stdout, "Address probed successfully\n" );
      break;
    case UVI_BERR :
      fprintf( stdout, "Bus Error\n" );
      break;
    default :
      fprintf( stdout, "Failed with system error %i\n", status );
      break;
    }

  if( VmeClose( fd )!= UVI_SUCCESS )
    {
      perror( "VmeClose" );
      return( -1 );
    }

  return( 0 );
}
