
#include "vmic_common_test.c"


/*
 * Read a value from the local slave window
 *
 * USAGE:
 * vspoke [-d data width] offset hexvalue ...
 *
 * data width - VMEbus access data width [8, 16, 32]
 *
 * offset - Number of bytes from SLAVE_BUFF_BASE to start from
 *
 * hexvalue - Hexidecimal byte value to be set at that offset.  Value may be
 *            a list of values to be set at successive byte locations from
 *            offset.
 *
 * RETURNS:
 *   0 on success or -1 on error.
 */
int main( int argc, char **argv )
{
  int            c, fd;
  unsigned       size, buffsize;
  void           *data;
  char           *buffer;
  UVI_UINT32     dwidth, offset, buffaddr, round;

  dwidth = DEFAULT_DWIDTH;

  while( ( c = getopt( argc, argv, "d:" ) ) != -1 )
    switch( c )
      {
      case 'd' :
	if( ( dwidth = strtodw( optarg ) ) < 0 )
	  {
	    fprintf( stderr, "Invalid data width\n" );
	    return( -1 );
	  }
	break;
      case '?' :
      default :
	fprintf( stderr, "vspeek [-d data width] offset [length]\n" );
	return( -1 );
      }

  if( argv[optind] )
    offset = strtoul( argv[optind++], NULL, 0 );
  else
    {
      fprintf( stderr, "No address specified\n" );
      return( -1 );
    }

  /*
   * Get data from argument list
   */
  size = ( argc - optind ) * ( dwidth / 8 );
  if( convert_hexdata( &data, size, dwidth, argv + optind ) < 0 )
    {
      perror( "convert_hexdata" );
      return( -1 );
    }

  DPRINTF(( stdout, "reading %d bytes, data width %d bits\n", size, dwidth ));

  if( ( fd = VmeOpen( ) ) < 0 )
    {
      perror( "VmeOpen" );
      return( -1 );
    }

  /*
   * Page align the mmap access
   */
  buffaddr = ( SLAVE_BUFF_BASE + offset ) & ~( PAGESIZE - 1 );
  round = ( SLAVE_BUFF_BASE + offset ) & ( PAGESIZE - 1 );
  buffsize = ( PAGESIZE + size + round + 1 )  & ~( PAGESIZE - 1 );

  DPRINTF(( stderr, "mapping 0x%x bytes of memory for poking at 0x%x\n",
	    buffsize, buffaddr ));

  /*
   * Map the buffer to the memory segment reserved at boot-up
   */
  if( MAP_FAILED == ( buffer = mmap( 0, buffsize,
				     PROT_READ | PROT_WRITE,
				     MAP_SHARED, fd,
				     buffaddr ) ) )
    {
      perror( "mmap" );
      VmeClose( fd );
      return( -1 );
    }

  vmemcpy( buffer + round, data, size, dwidth );

  if( munmap( buffer, buffsize ) < 0 )
    {
      perror( "munmap" );
      VmeClose( fd );
      return( -1 );
    }

  if( VmeClose( fd )!= UVI_SUCCESS )
    {
      perror( "VmeClose" );
      return( -1 );
    }

  return( 0 );
}

