/*
=================================================================================
                            COPYRIGHT NOTICE

    Copyright (C) 2000 VMIC
    International Copyright Secured.  All Rights Reserved.

---------------------------------------------------------------------------------
    Filename: uvi_api.h  Version: 1.1  Modification Date: 00/03/21 10:14:38
---------------------------------------------------------------------------------
    Description: Universe definitions/declarations for external use (Linux).
---------------------------------------------------------------------------------

    Revision History:
    Date         Who  Ver   Reason For Change
    -----------  ---  ----  -----------------------------------------------------
    21-MAR-2000  JW   1.1   Created.

=================================================================================
@(#)uvi_api.h 1.1 00/06/27 VMIC
*/

#ifndef __UVI_API_H
#define __UVI_API_H

#ifdef __cplusplus
extern "C" {
#endif

#define BUILD_API
#include "uvi_errno.h"
#include "uvi_universe.h"
#include "uvi_ioctl.h"

#define DEVICE "/dev/uvi"

/*****************************************************************************/

/* Define the device handle */

typedef struct uvihandle_s
{
    UVI_INT32  fd;          /* File descriptor returned by open(2) call */
    void       *MappedUvi;  /* Pointer to mmap'ed Uvi memory (if any)   */
    void       *ptr;        /* Usable pointer - has PAGE SIZE adjustment */
    UVI_UINT32 MappedSize;  /* Size of mmap'ed Uvi memory  */
    struct uviVmeWindowParameters Parameters;
    
} UVIHANDLE;

/*****************************************************************************/

/* File Open/Close prototypes */

int VmeOpen();
int VmeClose(UVI_UINT32);

/*****************************************************************************/

/* VME slave functions */

int OpenVmeSlave( UVI_UINT32 fdesc, UVI_UINT32 VmeBusAddr, UVI_UINT32 SuperUser,
                  UVI_UINT32 PgmData, UVI_UINT32 AddSpace, UVI_UINT32 WinSize,
                  UVI_UINT32 Offset, UVI_UINT32 WinNum );
int CloseVmeSlave( UVI_UINT32 fdesc, UVI_UINT32 WinNum );

/*****************************************************************************/

/* VME master functions */

int VmeGetPointer(UVI_UINT32 fdesc, UVI_UINT32 address,
                  UVI_UINT32 modifier, UVI_UINT32 size, UVIHANDLE *Handle);
int VmeReleasePointer(UVIHANDLE *Handle);

/*****************************************************************************/

/* debug functions */

int VmeSetDebugFlags(UVI_UINT32 fdesc, UVI_UINT32 Flags);
int VmeGetDebugFlags(UVI_UINT32 fdesc, UVI_UINT32 *Flags);

/*****************************************************************************/

/* VME master utility function */

int VmeProbeAddress(UVI_UINT32 fdesc, UVI_UINT32 Address, UVI_UINT32 Modifier,
                    UVI_UINT32 Width);

/*****************************************************************************/

/* VME DMA functions */
int VmeReleaseDma( UVI_UINT32 fdesc );
int VmeReqDma( UVI_UINT32 fdesc );
int VmeDmaIO( UVI_UINT32 fdesc, UVI_UINT32 VmeBusAddr, UVI_UINT32 AddrMod,
              UVI_UINT32 DmaSize, UVI_UINT32 DataWidth, UVI_BOOL ReadWrite,
              UVI_UINT32 BuffOffset, UVI_UINT32 * DmaCompStat ); 

/*****************************************************************************/

/* VME MailBox functions */

int RequestVmeMB( UVI_UINT32 fdesc, UVI_UINT32 MailboxNum );
int ReleaseVmeMB( UVI_UINT32 fdesc, UVI_UINT32 MailboxNum );
int CancelWaitVmeMB( UVI_UINT32 fdesc, UVI_UINT32 MailboxNum );
int WaitVmeMBInt( UVI_UINT32 fdesc, UVI_UINT32 VmeMailbox, UVI_UINT32 * MBdat );

/*****************************************************************************/

/* VME servicing interrupts functions */

int VmeWaitInt( UVI_UINT32 fdesc, UVI_UINT8 VmeLevel, UVI_UINT8 VmeVector,
            UVI_UINT32 * InterStatus );
int RequestVmeInt( UVI_UINT32 fdesc, UVI_UINT8 VmeLevel, UVI_UINT8 VmeVector );
int ReleaseVmeInt( UVI_UINT32 fdesc, UVI_UINT8 VmeLevel, UVI_UINT8 VmeVector );
int CancelWaitVmeInt( UVI_UINT32 fdesc, UVI_UINT8 VmeLevel, UVI_UINT8 VmeVector );

/*****************************************************************************/


#ifdef __cplusplus
}
#endif

#endif  /* __UVI_UNIVERSE_H */

















