/*
=================================================================================
                            COPYRIGHT NOTICE

    Copyright (C) 2000 VMIC
    International Copyright Secured.  All Rights Reserved.

---------------------------------------------------------------------------------
    Filename: uvi_errno.h  Version: 1.1  Modification Date: 00/03/21 10:14:39
---------------------------------------------------------------------------------
    Description: Universe module error codes
---------------------------------------------------------------------------------

    Revision History:
    Date         Who  Ver   Reason For Change
    -----------  ---  ----  -----------------------------------------------------
    21-MAR-2000  AE   1.1   Created.

=================================================================================
@(#)uvi_errno.h 1.1 00/06/27 VMIC
*/

#ifndef __UVI_ERRNO_H
#define __UVI_ERRNO_H

/* Driver debug message class definitions */

#define UVI_DBERROR         ( 1 << 0 )   /* Report critical errors            */
#define UVI_DBINIT          ( 1 << 1 )   /* Trace device probing and search   */
#define UVI_DBINTR          ( 1 << 2 )   /* Trace interrupt service           */
#define UVI_DBIOCTL         ( 1 << 3 )   /* Trace ioctl(2) system calls       */
#define UVI_DBMMAP          ( 1 << 4 )   /* Trace mmap(2) system calls        */
#define UVI_DBOPEN          ( 1 << 5 )   /* Trace open(2) system calls        */
#define UVI_DBCLOSE         ( 1 << 6 )   /* Trace close(2) system calls       */
#define UVI_DBREAD          ( 1 << 7 )   /* Trace read(2) system calls        */
#define UVI_DBWRITE         ( 1 << 8 )   /* Trace write(2) system calls       */
#define UVI_DBPEEK          ( 1 << 9 )   /* Trace peeks                       */
#define UVI_DBPOKE          ( 1 << 10 )  /* Trace pokes                       */
#define UVI_DBSTRAT         ( 1 << 11 )  /* Trace read/write strategy         */
#define UVI_DBTIMER         ( 1 << 12 )  /* Trace interval timer              */
#define UVI_DBTRACE         ( 1 << 13 )  /* Trace subroutine entry/exit       */
#define UVI_DBCALLBACK      ( 1 << 14 )  /* Trace user callback service       */
#define UVI_DBMUTEX         ( 1 << 15 )  /* Trace synchronization and locking */
#define UVI_DBEXTRA         ( 1 << 16 )  /* Trace extra                       */

#ifndef WHENDEBUG
#   define WHENDEBUG( x )   if( ( x ) & UviDebugFlags )
#endif

/* These error codes are made platform specific to allow error
   codes to reflect the style of the specific platform         */

/* No Current Error */
#define  UVI_SUCCESS              0

/* Function called is not currently implemented. */
#define  UVI_NOT_IMPLEMENTED      -1

/* An error occurred during a call to the driver. */
#define  UVI_DRIVER_ERROR         -2

/* PCI not configured */
#define  UVI_PCI_NOTCFG           -3

/* PCI bus not present */
#define  UVI_NO_PCI               -4

/* CPU not recognized as one of VMIC's */
#define  UVI_CPU_NOTREC           -5

/* Function failed for other OS defined error. 
   Use the OS's methods for getting the OS error. */
#define  UVI_OS_ERROR             -6

/* general mapping errors */
#define  UVI_MAP_ERROR            -7

/* Capability not supported by this particular Driver/Board */
#define  UVI_NOT_SUPPORTED        -8

/* Device not available */
#define  UVI_NO_UDEV              -9

/* NULL pointer error */
#define  UVI_NULL_PTR             -10

/* Invalid address modifier */
#define  UVI_INV_ADDRMOD          -11

/* Indicate a potential memory area conflict */
#define  UVI_MEM_CONFLICT         -12

/* No device available */
#define  UVI_NODEV_FOUND          -13

/* No device found */
#define  UVI_UNMAP_ERR            -14

/* No device window space available */
#define  UVI_NOSPC_AVAIL          -15

/* Device address conflict */
#define  UVI_DEVCONFLICT          -16

/* Ptr ok, but no device found */
#define  UVI_INV_WIDTH            -17

/* Parameter 2 to a function is wrong in some manner, 
   either a NULL pointer or an invalid value.         */
#define  UVI_BAD_PARAMETER_1      -18

/* Parameter 3 to a function is wrong in some manner, 
   either a NULL pointer or an invalid value.         */
#define  UVI_BAD_PARAMETER_2      -19

/* Parameter 4 to a function is wrong in some manner, 
   either a NULL pointer or an invalid value.         */
#define  UVI_BAD_PARAMETER_3      -20

/* Parameter 5 to a function is wrong in some manner, 
   either a NULL pointer or an invalid value.         */
#define  UVI_BAD_PARAMETER_4      -21

/* Parameter 6 to a function is wrong in some manner, 
   either a NULL pointer or an invalid value.         */
#define  UVI_BAD_PARAMETER_5      -22

/* Parameter 7 to a function is wrong in some manner, 
   either a NULL pointer or an invalid value.         */
#define  UVI_BAD_PARAMETER_6      -23

/* Parameter 8 to a function is wrong in some manner, 
   either a NULL pointer or an invalid value.         */
#define  UVI_BAD_PARAMETER_7      -24

/* Parameter 9 to a function is wrong in some manner, 
   either a NULL pointer or an invalid value.         */
#define  UVI_BAD_PARAMETER_8      -25

/* The lseek(2) operation preceding a read or write failed */
#define  UVI_LSEEK_ERROR          -26

/* The read(2) operation was not successful */
#define  UVI_READ_ERROR           -27

/* The write(2) operation was not successful */
#define  UVI_WRITE_ERROR          -28

/* General interrupt function returns */
#define UVI_INV_LVL_VECT          -29

/* Interrupt source check returns */
#define UVI_INTR_YOURS            0
#define UVI_INTR_NOT_YOURS        -30

/* Address probe results */
#define  UVI_BERR                 -31
#define  UVI_NOBERR               0

/* Image window specification error */
#define  UVI_WNDSPEC_ERROR        -32

/* Requested interrupt/vector already in use */
#define  UVI_INTR_IN_USE          -33

/* Interrupt handler registration failed */
#define  UVI_REGINTR_FAILED       -34

/* Device not open */
#define  UVI_NOT_OPEN             -35

/* Dma already running (busy) */
#define  UVI_DMA_BUSY             -36

/* Dma status error */
#define  UVI_DMA_STATUS_ERROR     -37

/* Dma status error */
#define  UVI_DMA_UNLOCK_ERROR     -38

/* Dma completion status errors */
/* VME bus error, local bus error, */
/* protocol error */
#define  UVI_DMA_VERR             -39
#define  UVI_DMA_LERR             -40
#define  UVI_DMA_PERR             -41

/* Mailbox status */
#define  UVI_MBOX_CANCELED        -42
#define  UVI_MBOX_ERROR           -43
#define  UVI_MBOX_BUSY            -44

/* VME interrupt status */
#define  UVI_VMEINT_NOT_USER      -45
#define  UVI_VMEINT_WAITING       -46
#define  UVI_VMEINT_NOTWAITING    -47

#endif  /* __UVI_ERRNO_H */
