/*
=================================================================================
                            COPYRIGHT NOTICE

    Copyright (C) 2000 VMIC
    International Copyright Secured.  All Rights Reserved.

---------------------------------------------------------------------------------
    Filename: uvi_moduledefs.h  Version: 1.1  Modification Date: 00/03/21 10:14:40
---------------------------------------------------------------------------------
    Description: Universe module internal definitions (Linux).
---------------------------------------------------------------------------------

    Revision History:
    Date         Who  Ver   Reason For Change
    -----------  ---  ----  -----------------------------------------------------
    21-MAR-2000  AE   1.1  Created.

=================================================================================
@(#)uvi_moduledefs.h 1.1 00/06/27 VMIC
*/

#ifndef __UVIMODULE_DEFS_H
#define __UVIMODULE_DEFS_H

#ifdef __cplusplus
extern "C" {
#endif

/* Common Headers */
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <asm/io.h>
#include <asm/errno.h>
#include <asm/uaccess.h>
#include <asm/page.h>
#include <zconf.h>

#include "uvi_universe.h"
#include "uvi_errno.h"

#define MODULE_NAME                 "UnivModule"

/* Universe PCI registers offsets */
#define PCI_ID                      0x000

#define UNIV_VID                    0x10E3
#define UNIV_DID                    0x0000
#define PLX_VID                     0x114A
#define PLX_DID                     0x0001

#define PCI_CSR                     0x004

#define PCI_BUS_MSTR_EN             0x00000004

#define PCI_CLASS                   0x008

/* Universe master control register offset */
#define MAST_CTL                    0x400

/* Universe master control reg bit vals */
#define MAXRTRY( x )                ( ( (x) & 0x3C0 ) << 22 )
#define PWON_CNT_128                0x00000000
#define PWON_CNT_256                0x01000000
#define PWON_CNT_512                0x02000000
#define PWON_CNT_1024               0x03000000
#define PWON_CNT_2048               0x04000000
#define PWON_CNT_4096               0x05000000
#define VRL_0                       0x00000000
#define VRL_1                       0x00400000
#define VRL_2                       0x00800000
#define VRL_3                       0x00C00000
#define VRM_DEMAND                  0x00000000
#define VRM_FAIR                    0x00200000
#define	VREL_ON_REQ                 0x00100000
#define VREL_WHEN_DONE              0x00000000
#define VOWN_RELEASE_BUS            0x00000000
#define VOWN_ACQ_AND_HOLD_BUS       0x00080000
/* #define VINT_OWN_ACK_MASK         0x00040000 */
#define VOWN_ACK_NOT_OWNED          0x00000000
#define VOWN_ACK_OWNED_AND_HELD     0x00040000
#define PABS_32                     0x00000000
#define PABS_64                     0x00001000
#define BUS_NO( x )                 ( (x) & 0xFF )

/* Universe misc. control reg. offset */
#define MISC_CTRL                   0x404

/* Universe misc. control reg bit values */
#define VBTO_DISABLE                0x00000000
#define VBTO_16US                   0x10000000
#define VBTO_32US                   0x20000000
#define VBTO_64US                   0x30000000
#define VBTO_128US                  0x40000000
#define VBTO_256US                  0x50000000
#define VBTO_512US                  0x60000000
#define VBTO_1024US                 0x70000000
#define VARB_RNDRBN                 0x00000000
#define VARB_PRIORITY               0x04000000
#define VARB_TOUT_DISABL            0x00000000
#define VARB_TOUT_16US              0x01000000
#define	VARB_TOUT_256US             0x02000000
#define SW_LRST                     0x00800000
#define SW_SYSRST                   0x00400000
#define BI_MODE_EN                  0x00100000
#define BI_MODE_DIS                 0xFFEFFFFF
#define ENGBI_MODE                  0x00080000
#define RESCIND_DTACK_EN            0x00040000
#define SYSCON                      0x00020000
#define V64AUTO_ID                  0x0000FF00

/* Register offsets for PLX device */
#define PLX_COMM                    0x00
#define PLX_B_INT_STATUS            0x04
#define PLX_B_INT_MASK              0x08
#define PLX_B_ADDRESS               0x0C

/*
** Location of the command register & the 'board id' register for non-PLX
** VMIC CPU's.
*/
#define NOTPLX_CMDREG_ADDR          0xD800E
#define NOTPLX_BRDID_ADDR           0xD8016

/* Bit values for PLX and not-PLX registers */
#define MEC_PLX                     0x00000001
#define SEC_PLX                     0x00000002
#define ABLE_PLX                    0x00000004
#define BTO_EN_PLX                  0x00000008
#define BTO_16US_PLX                0x00000000
#define BTO_64US_PLX                0x00000010
#define BTO_256US_PLX               0x00000020
#define BTO_1MS_PLX                 0x00000030
#define BERRIM_INTA_PLX             0x00000000
#define BERRIM_NMI_PLX              0x00000040
#define MB_M0_EN_PLX                0x00000080
#define VME_EN_PLX                  0x00000800
#define VME_DIS_PLX                 0x0000F7FF

#define BERR_S_PLX                  0x00000001

#define MEC_NOTPLX                  0x00000001
#define SEC_NOTPLX                  0x00000002
#define ABLE_NOTPLX                 0x00000004
#define BTO_EN_NOTPLX               0x00000008
#define BTO_16US_NOTPLX             0x00000000
#define BTO_64US_NOTPLX             0x00000010
#define BTO_256US_NOTPLX            0x00000020
#define BTO_1MS_NOTPLX              0x00000030
#define BERRI_EN_NOTPLX             0x00000040
#define BERRST_CLR_NOTPLX           0x00000080
#define WTDSYS_NOTPLX               0x00000100
#define VME_EN_NOTPLX               0x00000800
#define VME_DIS_NOTPLX              0x0000F7FF

/* Universe 1/2 PCI interrupt register defines */
#define LINT_EN                     0x300
#define LINT_STAT                   0x304

#define LINT_MB0                    0x10000
#define LINT_MB1                    0x20000
#define LINT_MB2                    0x40000
#define LINT_MB3                    0x80000

#define LINT_DMA                    0x100
#define LINT_VME_INT                0xFE
#define LINT_EN_VIRQ( x )           ( (UVI_UINT32)( 1 << (x) ) )
#define LINT_DIS_VIRQ( x )          ~LINT_EN_VIRQ( (x) )
#define LINT_CLR_VIRQ( x )          ( (UVI_UINT32)( 1 << (x) ) )

#define LINT_DIS_ALL_U1             0xFFFF0800  /* and'd with en reg */
#define LINT_DIS_ALL_U2             0xFF000800  /* and'd with en reg */
#define LINT_CLR_ALL_U1             ~LINT_DIS_ALL_U1  /* or'd with stat reg */
#define LINT_CLR_ALL_U2             ~LINT_DIS_ALL_U2  /* or'd with stat reg */

/* PCI interrupt mapping */
#define LINT_MAP0                   0x308
#define LINT_MAP1                   0x30C
#define LINT_MAP2                   0x340

/* Mailbox regs */
#define VRAI_CTL                    0xF70
#define VRAI_BS                     0xF74
#define MBOX0                       0x348
#define MBOX1                       0x34C
#define MBOX2                       0x350
#define MBOX3                       0x354

#define VIRQ7_TO_PCI( x )           ( (UVI_UINT32)( ( (x) & 0x07 ) <<28 ) )
#define VIRQ6_TO_PCI( x )           ( (UVI_UINT32)( ( (x) & 0x07 )<<24) )
#define VIRQ5_TO_PCI( x )           ( (UVI_UINT32)( ( (x) & 0x07 )<<20) )
#define VIRQ4_TO_PCI( x )           ( (UVI_UINT32)( ( (x) & 0x07)<<16) )
#define VIRQ3_TO_PCI( x )           ( (UVI_UINT32)( ( (x) & 0x07)<<12) )
#define VIRQ2_TO_PCI( x )           ( (UVI_UINT32)( ( (x) & 0x07)<<8) )
#define VIRQ1_TO_PCI( x )           ( (UVI_UINT32)( ( (x) & 0x07)<<4) )
#define VOWN_TO_PCI( x )            ( (UVI_UINT32)( ( (x) & 0x07)<<0) )
#define ACFAIL_TO_PCI( x )          ( (UVI_UINT32)( ( (x) & 0x07)<<28) )
#define SYSFAIL_TO_PCI( x )         ( (UVI_UINT32)( ( (x) & 0x07)<<24) )
#define SW_INT_TO_PCI( x )          ( (UVI_UINT32)( ( (x) & 0x07)<<20) )
#define SW_IACK_TO_PCI( x )         ( (UVI_UINT32)( ( (x) & 0x07)<<16) )
#define VERR_TO_PCI( x )            ( (UVI_UINT32)( ( (x) & 0x07)<<8) )
#define LERR_TO_PCI( x )            ( (UVI_UINT32)( ( (x) & 0x07)<<4) )
#define DMA_TO_PCI( x )             ( (UVI_UINT32)( (x) & 0x07 ) )

/* Universe 1/2 VME interrupt register defines */
#define VINT_EN                     0x310
#define VINT_STAT                   0x314

#define VINT_EN_SW_INT              0x00001000  /* or'd */
#define VINT_DIS_SW_INT             0xFFFFEFFF  /* and'd */
#define VINT_CLR_SW_INT             0x00001000  /* or'd */
#define VINT_RST_SW_INT             0xFFFFEFFF  /* and'd */

#define VINT_DIS_ALL_U1             0xFFFFE800  /* and'd */
#define VINT_DIS_ALL_U2             0x01F0E800  /* and'd */
#define VINT_CLR_ALL_U1             ~VINT_DIS_ALL_U1  /* or'd */
#define VINT_CLR_ALL_U2             ~VINT_DIS_ALL_U2  /* or'd */

/* VME interrupt mapping */
#define VINT_MAP0                   0x318
#define VINT_MAP1                   0x31C

#define SW_INT_TO_VME( x )          ( (UVI_UINT32)( ( (x) & 0x07 ) << 16 ) )

#define CLEAR_SW_INT_MAP            ( ~SW_INT_TO_VME( 7 ) )

/* VME bus status id registers */
#define STATID_LOCAL                0x323
#define V_STATID_BASE               0x324
#define V_STATID_OFFS               0x004

#define V1_STATID                   0x324
#define V2_STATID                   0x328
#define V3_STATID                   0x32C
#define V4_STATID                   0x330
#define V5_STATID                   0x334
#define V6_STATID                   0x338
#define V7_STATID                   0x33C

/* Universe DMA registers */
#define DCTL                        0x200
#define DTBC                        0x204
#define DLA                         0x208
#define DVA                         0x210
#define DCPP                        0x218
#define DGCS                        0x220
#define D_LLUE                      0x224

/* PCI target image registers */
#define LSI0_CTL                    0x100
#define LSI0_BS                     0x104
#define LSI0_BD                     0x108
#define LSI0_TO                     0x10C
#define LSI1_CTL                    0x114
#define LSI1_BS                     0x118
#define LSI1_BD                     0x11C
#define LSI1_TO                     0x120
#define LSI2_CTL                    0x128
#define LSI2_BS                     0x12C
#define LSI2_BD                     0x130
#define LSI2_TO                     0x134
#define LSI3_CTL                    0x13C
#define LSI3_BS                     0x140
#define LSI3_BD                     0x144
#define LSI3_TO                     0x148
#define LSI4_CTL                    0x1A0
#define LSI4_BS                     0x1A4
#define LSI4_BD                     0x1A8
#define LSI4_TO                     0x1B0
#define LSI5_CTL                    0x1B4
#define LSI5_BS                     0x1B8
#define LSI5_BD                     0x1BC
#define LSI5_TO                     0x1C0
#define LSI6_CTL                    0x1C8
#define LSI6_BS                     0x1CC
#define LSI6_BD                     0x1D0
#define LSI6_TO                     0x1D4
#define LSI7_CTL                    0x1DC
#define LSI7_BS                     0x1E0
#define LSI7_BD                     0x1E4
#define LSI7_TO                     0x1E8

#define SLSI                        0x188

/*
** PCI target image base/bound values.  PCI target images 0 & 4( U2 ) have
** a 4Kb resolution.  Images 1,2,3,5,6, & 7 have a 64Kb resolution.
*/
#define SPECWND_A24UD_BASE          0x40000000
#define SPECWND_A16UD_BASE          0x40FF0000
#define SPECWND_A24SD_BASE          0x41000000
#define SPECWND_A16SD_BASE          0x41FF0000
#define SPECWND_A24UD_BASE2         0x42000000
#define SPECWND_A16UD_BASE2         0x42FF0000 
#define SPECWND_A24SD_BASE2         0x43000000
#define SPECWND_A16SD_BASE2         0x43FF0000

#define V_A16_BOUND                 0x0000FFFF
#define V_A24_BOUND                 0x00FFFFFF

#define WINDOW0_A24UD_BASE          0x44000000
#define WINDOW0_A24UD_BOUND         0x44FFFFFF

#define WINDOW1_A24SD_BASE          0x45000000
#define WINDOW1_A24SD_BOUND         0x45FF0000

#define WINDOW2_A32UD_BASE          0x50000000
#define WINDOW2_A32UD_BOUND         0x7FFF0000

#define WINDOW3_A32SD_BASE          0x80000000
#define WINDOW3_A32SD_BOUND         0xAFFF0000

/*
** Translation offset register defines for A16/A24 space.  These are here for
** consistent readability in the code but are not to be modified by the user.
** There is no need to change these values.
*/
#define VME_A16UD_BASE              0x0
#define VME_A16SD_BASE              0x0
#define VME_A24UD_BASE              0x0
#define VME_A24SD_BASE              0x0

/* PCI target image control register bit values */
#define TRGT_IMAGE_EN               0x80000000
#define TRGT_IMAGE_DIS              0x7FFFFFFF
#define TRGT_PWEN                   0x40000000
#define TRGT_VDW_8                  0x00000000
#define TRGT_VDW_16                 0x00400000
#define TRGT_VDW_32                 0x00800000
#define TRGT_VDW_64                 0x00C00000
#define TRGT_VAS_A16                0x00000000
#define TRGT_VAS_A24                0x00010000
#define TRGT_VAS_A32                0x00020000
#define TRGT_VAS_CRCSR              0x00050000
#define TRGT_VAS_USER1              0x00060000
#define TRGT_VAS_USER2              0x00070000
#define TRGT_PGM_PROGRAM            0x00004000
#define TRGT_PGM_DATA               0x00000000
#define TRGT_SUPER_P                0x00001000
#define TRGT_SUPER_NP               0x00000000
#define TRGT_VCT_BLT_EN             0x00000100
#define TRGT_LAS_IO                 0x00000001
#define TRGT_LAS_MEM                0x00000000

/* Special PCI target image bit values */
#define SLSI_TRGT_IMAGE_EN          0x80000000
#define SLSI_TRGT_IMAGE_DIS         0x7FFFFFFF
#define SLSI_TRGT_PWEN              0x40000000
#define SLSI0_TRGT_VDW_16           0x00000000
#define SLSI0_TRGT_VDW_32           0x00100000
#define SLSI1_TRGT_VDW_16           0x00000000
#define SLSI1_TRGT_VDW_32           0x00200000
#define SLSI2_TRGT_VDW_16           0x00000000
#define SLSI2_TRGT_VDW_32           0x00400000
#define SLSI3_TRGT_VDW_16           0x00000000
#define SLSI3_TRGT_VDW_32           0x00800000
#define SLSI0_TRGT_PGM_PRGM         0x00001000
#define SLSI0_TRGT_PGM_DATA         0x00000000
#define SLSI1_TRGT_PGM_PRGM         0x00002000
#define SLSI1_TRGT_PGM_DATA         0x00000000
#define SLSI2_TRGT_PGM_PRGM         0x00004000
#define SLSI2_TRGT_PGM_DATA         0x00000000
#define SLSI3_TRGT_PGM_PRGM         0x00008000
#define SLSI3_TRGT_PGM_DATA         0x00000000
#define SLSI0_TRGT_SUPER_P          0x00000100
#define SLSI0_TRGT_SUPER_NP         0x00000000
#define SLSI1_TRGT_SUPER_P          0x00000200
#define SLSI1_TRGT_SUPER_NP         0x00000000
#define SLSI2_TRGT_SUPER_P          0x00000400
#define SLSI2_TRGT_SUPER_NP         0x00000000
#define SLSI3_TRGT_SUPER_P          0x00000800
#define SLSI3_TRGT_SUPER_NP         0x00000000
#define SLSI_TRGT_BS                0x00000040 /* base @ 40000000 */
#define SLSI_TRGT_LAS_IO            0x00000001
#define SLSI_TRGT_LAS_MEM           0x00000000

/* VMe target image registers */
#define VSI0_CTL                    0xF00
#define VSI0_BS                     0xF04
#define VSI0_BD                     0xF08
#define VSI0_TO                     0xF0C
#define VSI1_CTL                    0xF14
#define VSI1_BS                     0xF18
#define VSI1_BD                     0xF1C
#define VSI1_TO                     0xF20
#define VSI2_CTL                    0xF28
#define VSI2_BS                     0xF2C
#define VSI2_BD                     0xF30
#define VSI2_TO                     0xF34
#define VSI3_CTL                    0xF3C
#define VSI3_BS                     0xF40
#define VSI3_BD                     0xF44
#define VSI3_TO                     0xF48
#define VSI4_CTL                    0xF90
#define VSI4_BS                     0xF94
#define VSI4_BD                     0xF98
#define VSI4_TO                     0xF9C
#define VSI5_CTL                    0xFA4
#define VSI5_BS                     0xFA8
#define VSI5_BD                     0xFAC
#define VSI5_TO                     0xFB0
#define VSI6_CTL                    0xFB8
#define VSI6_BS                     0xFBC
#define VSI6_BD                     0xFC0
#define VSI6_TO                     0xFC4
#define VSI7_CTL                    0xFCC
#define VSI7_BS                     0xFD0
#define VSI7_BD                     0xFD4
#define VSI7_TO                     0xFD8


/* VME slave image control bits */
#define VSI_IMAGE_EN                0x80000000
#define VSI_CTL_PGM                 0x00800000  /* program */
#define VSI_CTL_DAT                 0x00400000  /* data */
#define VSI_CTL_PD                  0x00C00000  /* both program and data */
#define VSI_CTL_SUPER               0x00200000  /* supervisory */
#define VSI_CTL_USER                0x00100000  /* nonprivileged */
#define VSI_CTL_SU                  0x00300000  /* both supervisory and nonprivileged */
#define VSI_CTL_A32                 0x00020000  /* A32 space */
#define VSI_CTL_A24                 0x00010000  /* A24 space */
#define VSI_CTL_A16                 0x00000000  /* A16 space */

/* */
#define UVI_INTRID                  0x00757669 /* ascii u - v - i */

/* Maximum # of devices that information will be stored for. */
#define MAX_DEVICES                 128

/* Interrupt flag bits. */
#define VECTOR_IN_USE               0x01
#define INTR_PENDING                0x02
#define USER_OWNS_INT               0x10
#define USER_READY_INT              0x20
#define CANCEL_USER_OWNS_INT        0x80
#define IACK_BUS_ERROR              0x100

/* DMA flag bits. */
#define DMA_IN_USE                  0x01
#define USER_OWNS_DMA               0x10
#define USER_READY_DMA              0x20
#define CANCEL_USER_WAIT_DMA        0x80

/* Base register pointer declarations */
volatile unsigned char *UnivBaseAddr,
                       *PlxMappedAddr;

/* PCI devices pointers */
struct pci_dev *AllPciDevices,
               *UviDevice,
               *UviPlxDevice,
               *PciDevice;

UVI_UINT16 IsaIdRegVal,
           IsaCmdRegVal;

UVI_UINT32 UdeviceOk;

/* Interrupt data. */
struct uviInterruptData
{
    UVI_UINT8 IntrFlags, /* kernel/user, sleeping, signal, etc. */
              QueueCnt;
};
    
#ifdef ALLOC_DATA
  struct uviInterruptData IntTable[MAX_VINT_LVLS][MAX_VINT_VECTORS];

  UVI_UINT32 UviDebugFlags = 0,
             VmeBerrStatus,
             MaxA16UDSpace,
             MaxA16SDSpace,
             MaxA24UDSpace,
             MaxA24SDSpace,
             MaxA32UDSpace,
             MaxA32SDSpace,
             *DevId;

  UVI_INT32  UnivFound,
             Univ1Found,
             UnivRevId,
             PlxFound,
             UnivIrq;
/*
  spinlock_t WndOp_SpinLock;
  spinlock_t IrqOp_SpinLock;
*/
#else
  extern struct uviInterruptData IntTable[MAX_VINT_LVLS][MAX_VINT_VECTORS];

  extern UVI_UINT32 UviDebugFlags,
                    VmeBerrStatus,
                    MaxA16UDSpace,
                    MaxA16SDSpace,
                    MaxA24UDSpace,
                    MaxA24SDSpace,
                    MaxA32UDSpace,
                    MaxA32SDSpace,
                    *DevId;

  extern UVI_INT32  UnivFound,
                    Univ1Found,
                    UnivRevId,
                    PlxFound,
                    UnivIrq;
/*
  extern spinlock_t WndOp_SpinLock;
  extern spinlock_t IrqOp_SpinLock;
*/
#endif

UVI_INT32 uviInitDev( void );  /* uvi - universe vme interface */
UVI_INT32 uviGetPciRegions( void );
UVI_INT32 uviInitVmeWindows( void );
UVI_INT32 uviInitMailboxes( void );
UVI_INT32 uviInstallUniverseInterruptHandler( void );
UVI_INT32 uviRemoveUniverseInterruptHandler( void );
void uviUniverseIntrHandlr( int, void *, struct pt_regs * );


/* Proc Page stuff */
#ifndef UVI_DYNAMIC_MAJOR_NUMBER
  #define UVI_DYNAMIC_MAJOR_NUMBER    0
#endif

extern struct proc_dir_entry uviProcEntry;


/* Standard file operations entry points */
int uvi_open(struct inode *inode, struct file *filp);
int uvi_ioctl( struct inode *inode, struct file *filp, unsigned int cmd, 
    unsigned long arg );

int uvi_release(struct inode *inode, struct file *filp);
int uvi_mmap(struct file *filp, struct vm_area_struct *vma);
   
#ifdef ALLOC_DATA

  /* Our device name.  Make sure DEVNAME_SIZE is correct. */
  const char *devname = "uvi";

  /* Dynamically assign a major device number */
  UVI_INT32 major_num = UVI_DYNAMIC_MAJOR_NUMBER; 

  /* Our file operations table */

  struct file_operations uvi_fops = 
  {
      NULL,           /* llseek               */
      NULL,           /* read                 */
      NULL,           /* write                */
      NULL,           /* readdir              */
      NULL,           /* poll                 */
      uvi_ioctl,      /* ioctl                */
      uvi_mmap,       /* mmap                 */
      uvi_open,       /* open                 */
      NULL,           /* flush                */
      uvi_release,    /* release              */
      NULL,           /* fsync                */
      NULL,           /* fasync               */
      NULL,           /* check_media_change   */
      NULL,           /* revalidate           */
      NULL            /* lock                 */
  };

#else
  extern char *devname;
  extern UVI_INT32 major_num;
  extern struct file_operations uvi_fops;
#endif

#define DEVNAME_SIZE 3


#ifdef __cplusplus
}
#endif

#endif  /* __UVIMODULE_DEFS_H */

















