/*
=================================================================================
                            COPYRIGHT NOTICE

    Copyright (C) 2000 VMIC
    International Copyright Secured.  All Rights Reserved.

---------------------------------------------------------------------------------
    Filename: uvi_universe.h  Version: 1.1  Modification Date: 00/03/21 10:14:40
---------------------------------------------------------------------------------
    Description:  Definitions/declarations for stacked module use (Linux).
---------------------------------------------------------------------------------

    Revision History:
    Date         Who  Ver   Reason For Change
    -----------  ---  ----  -----------------------------------------------------
    21-MAR-2000  AE   1.1   Created.

=================================================================================
@(#)uvi_universe.h 1.1 00/06/27 VMIC
*/

#ifndef __UVI_UNIVERSE_H
#define __UVI_UNIVERSE_H

#ifdef __cplusplus
extern "C" {
#endif

#include <asm/spinlock.h>

/*******************************************************************************/

/*
** Provide Type Definitions for basic types for this platform (LINUX)
*/
typedef unsigned char           UVI_UINT8;
typedef unsigned short int      UVI_UINT16;
typedef unsigned int            UVI_UINT32;
typedef unsigned long long int  UVI_UINT64;

typedef signed char             UVI_INT8;
typedef signed short int        UVI_INT16;
typedef signed int              UVI_INT32;
typedef signed long long int    UVI_INT64;

typedef unsigned char           UVI_BOOL;

/*
** We also use void * and char *, but these are
** C standard types, and do not need to be declares
*/


/*******************************************************************************/

/* Add base constants for the Boolean */
static const UVI_BOOL           UVI_TRUE = 1;
static const UVI_BOOL           UVI_FALSE = 0;

/*******************************************************************************/

/* Define the scalar types */
typedef UVI_INT32               UVIRESULT;  /* Return codes for functions   */


/*******************************************************************************/

#define UVI_BYTE         8
#define UVI_WORD         16
#define UVI_LWORD        32

#define UVI_KERNEL       0
#define UVI_USER         1

#define V_A16UD          0x29
#define V_A16SD          0x2D
#define V_A24UD          0x39
#define V_A24SD          0x3D
#define V_A24UP          0x3A
#define V_A24SP          0x3E
#define V_A32UD          0x09
#define V_A32SD          0x0D
#define V_A32UP          0x0A
#define V_A32SP          0x0E

#define V_A16            0x0
#define V_A24            0x1
#define V_A32            0x2

#define V_D8             0x0
#define V_D16            0x1
#define V_D32            0x2
#define V_D64            0x3

#define DMA_WR		 0x1
#define DMA_RD		 0x0

/* VME slave image control bits */
#define VSI_CTL_PGM	 0x00800000	/* program */
#define VSI_CTL_DAT	 0x00400000	/* data */
#define VSI_CTL_PD	 0x00C00000	/* both program and data */
#define VSI_CTL_SUPER	 0x00200000	/* supervisory */
#define VSI_CTL_USER	 0x00100000	/* non-privileged */
#define VSI_CTL_SU	 0x00300000	/* both supervisory and non-privileged */
#define VSI_CTL_A32	 0x00020000	/* A32 address space */
#define VSI_CTL_A24	 0x00010000	/* A24 address space */
#define VSI_CTL_A16	 0x00000000	/* A16 address space */

/*
** Translation offset register defines for A32 space.  These may be modified
** by the user to meet board addressing needs.  These are initially set to give
** an offset of 0 on the VME bus.
*/
#define VME_A32UD_BASE              0x0
#define VME_A32SD_BASE              0x0

/*****************************************************************************/

/* VME Mailbox setup. The Mailboxes are setup after the master windows
** have been setup. Mailboxes are not enabled if MB_REG_EN is equal to 0.
** There should be only one line uncommented per section below.
** NOTE: MB_REG_ADD is the VME base address for all of the Tundra Universe
**       registers. When enabled, the user has VME access to all of the
**       4 Kbyte of Tundra Universe registers. The 4 32 Bit Mailboxes start
**       at offset 0x348 from the base address. Access to any other registers
**       can cause undetermined results.
** DEFAULT: Enabled / A16 / Non-privileged / Data access at VME Address 0
*/
/****************************************************/
/* Mailboxes enabled / not enabled section */
/****************************************************/
               /* mail boxes will be enabled */
#define MB_REG_EN                   0x80000000  
               /* mail boxes not enabled */
/* #define MB_REG_EN                   0x00000000 */

/****************************************************/
/* Mailboxes set for Program / Data / Both section */
/****************************************************/
               /* data access */
#define MB_REG_PD                   0x00400000
               /* program access */
/* #define MB_REG_PD                   0x00800000 */
               /* both program and data access */
/* #define MB_REG_PD                   0x00C00000 */

/****************************************************/
/* Mailboxes set for Non-privileged / Supervisory / Both section */
/****************************************************/
               /* Non-privileged access */
#define MB_REG_NPS                  0x00100000
               /* Supervisory access */
/* #define MB_REG_NPS                  0x00200000 */
               /* Both Non-privileged and Supervisory access */
/* #define MB_REG_NPS                  0x00300000 */

/****************************************************/
/* Mailboxes set for A16 / A24 / A32 section */
/****************************************************/
               /* A16 space */
#define MB_REG_SPACE                0x00000000
               /* A24 space */
/* #define MB_REG_SPACE                0x00010000 */
               /* A32 space */
/* #define MB_REG_SPACE                0x00020000 */

/* Tundra Universe register access base address */
               /* A16 space, (64K Max) */
#define  MB_REG_ADD                 0x0000
               /* A24 space, (16M Max) */
/* #define  MB_REG_ADD                 0x000000 */
               /* A32 space, (4Gig) */
/* #define  MB_REG_ADD                 0x00000000 */

/*****************************************************************************/

/* level/vector interrupt limits for module/user functions */
#define MAX_VINT_LVLS               7
#define MAX_VINT_VECTORS            256

spinlock_t IrqOpSpinLock;

struct wait_queue *wq; /* interrupt testing */

struct uviVmeWindowParameters
{
  UVI_UINT32 SysModeFlag;
  UVI_UINT32 ReqWndSize;
  UVI_UINT32 AddrMod;
  UVI_UINT32 VmeBusAddr;
  UVI_UINT32 PhysAddr;
  volatile unsigned char *PtrToVmeWindow;
};

/* VME interrupt structure */
struct uviInterruptParameters
{
  UVI_UINT32 Fdesc;
  UVI_UINT8 IntLevel;
  UVI_UINT8 IntVector;
  UVI_UINT32 IntStatus;
};

/* mailbox structure */
struct uviMailboxParameters
{
  UVI_UINT32 Fdesc;
  UVI_UINT32 MailboxNum;
  UVI_UINT32 MailboxDat;
};

/*
added 7/25/00 sjl - use UVI_DMAWR UVI_DMARD for DmaRdWr; 
the beginning DMA address BuffOffset is the offset from DMA_BUFF_BASE
(linux mem=31M reserves 1 Meg of a 32 Meg system)
*/
#define	DMA_BUFF_BASE		0x07F00000	/* DMA buffer base address set to 127 Meg (128 Meg - 1 Meg) */

struct uviVmeDmaParameters
{
  UVI_UINT32 Fdesc;
  UVI_UINT32 DmaCompStat;
  UVI_BOOL   DmaRdWr;
  UVI_UINT32 DmaSize;
  UVI_UINT32 AddrMod;
  UVI_UINT32 VmeBusAddr;
  UVI_UINT32 DataWidth;
  UVI_UINT32 DmaOffset;
};

struct uviVmeDmaSetup
{
  UVI_UINT32 Fdesc;
  UVI_UINT32 DmaCompStat;
  void (*CallbackFunctionPtr) ( UVI_UINT32 );  /* user call back function 
                                                 with dma status arg */
};

#define	SLAVE_BUFF_BASE		0x07F00000	/* SLAVE buffer base address set to 127 Meg */

struct uviVmeSlaveWinParameters
{
  UVI_UINT32 SysModeFlag;
  UVI_UINT32 Fdesc;
  UVI_UINT32 VmeBusAddr;
  UVI_UINT32 SuperUser;
  UVI_UINT32 PgmData;
  UVI_UINT32 AddSpace;
  UVI_UINT32 WinSize;
  UVI_UINT32 Offset;
  UVI_UINT32 WinNum;
};

struct uviVmeSlaveCloseParams
{
  UVI_UINT32 SysModeFlag;
  UVI_UINT32 Fdesc;
  UVI_UINT32 WinNum;
};

typedef struct uviEventInfo
{
  UVI_INT32 VmeIntLvl;
  UVI_INT32 VmeIntVector;
  UVI_UINT32 TimeOut;
} UVIEVENTINFO;

#ifndef BUILD_API
    struct uviVmeWindowParameters VmeWindowParameters;
#endif

UVI_INT32 uviInitVme( void );

UVI_INT32 uviGetVmeWindow( struct uviVmeWindowParameters * );

UVI_INT32 uviReleaseVmeWindow( struct uviVmeWindowParameters * );

UVI_INT32 uviProbeAddress( struct uviVmeWindowParameters *,
                           UVI_INT16 );

UVI_INT32 uviGetUniverseInterruptLevel( UVI_UINT32 * );

UVI_INT32 uviEnableInterrupt( UVI_INT32,
                              UVI_INT32 );

UVI_INT32 uviCheckIntSource( UVI_INT32,
                             UVI_INT32 );

UVI_INT32 uviReArmInterrupt( UVI_INT32,
                             UVI_INT32 );

UVI_INT32 uviDisableInterrupt( UVI_INT32,
                               UVI_INT32 );

UVI_INT32 uviEnableDma( UVI_UINT32,
                        UVI_UINT32,
                        volatile UVI_UINT8 ** );

UVI_INT32 uviDisableDma( volatile UVI_UINT8 * );

UVI_INT32 uviReleaseUseDma( void );

UVI_INT32 uviRequestUseDma( void );

UVI_INT32 uviWaitDmaInt( UVI_UINT32 * );

UVI_INT32 uviRequestUseVmeInt( UVI_INT32, UVI_INT32 );

UVI_INT32 uviRequestUseMB( UVI_INT32 );

UVI_INT32 uviReleaseUseMB( UVI_INT32 );

UVI_INT32 uviCancelMB( UVI_INT32 );

UVI_INT32 uviEnableMB( UVI_INT32, UVI_UINT32 *, UVI_INT32 );

UVI_INT32 uviAccessDmaBuffer( UVI_UINT32,
                              UVI_UINT32,
                              UVI_INT32,
                              UVI_INT32,
                              UVI_UINT8,
                              UVI_BOOL,
                              UVI_INT32 *,
                              UVI_INT32 );

UVI_INT32 uviConfigSlaveWin( volatile struct uviVmeSlaveWinParameters * );

UVI_INT32 uviCloseSlaveWin( volatile struct uviVmeSlaveCloseParams * );

UVI_INT32 uviEnableDmaInterrupt( void );

UVI_INT32 uviDisableDmaInterrupt( void );

UVI_INT32 uviRequestUseVmeInt( UVI_INT32, UVI_INT32 );

UVI_INT32 uviReleaseVmeInt( UVI_INT32, UVI_INT32 );

UVI_INT32 uviEnableVmeInt( volatile struct uviInterruptParameters * );



#ifdef __cplusplus
}
#endif

#endif  /* __UVI_UNIVERSE_H */
