/*
=================================================================================
                            COPYRIGHT NOTICE

    Copyright (C) 2000 VMIC
    International Copyright Secured.  All Rights Reserved.

---------------------------------------------------------------------------------
    Filename: driver_entry.c  Version: 1.1  Modification Date: 00/03/21 10:15:01
---------------------------------------------------------------------------------
    Description: Standard file operations entry points for the Universe driver 
        (Linux)
---------------------------------------------------------------------------------

    Revision History:
    Date         Who  Ver   Reason For Change
    -----------  ---  ----  -----------------------------------------------------
    21-MAR-2000  JW   1.1   Created.


=================================================================================
*/

#ident "@(#)driver_entry.c 1.1 00/06/27 VMIC"

#define __NO_VERSION__

#include <asm/io.h>
#include <asm/uaccess.h>
#include "uvi_ioctl.h"
#include "uvi_moduledefs.h"

/*** Local prototypes **********************************************************/

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __cplusplus
}
#endif

atomic_t open_count = ATOMIC_INIT(0);

extern atomic_t dma_lock;
extern atomic_t MB0_lock;
extern atomic_t MB1_lock;
extern atomic_t MB2_lock;
extern atomic_t MB3_lock;
extern UVI_UINT8 UserOwnDma;
extern UVI_UINT8 UserOwnMB0;
extern UVI_UINT8 UserOwnMB1;
extern UVI_UINT8 UserOwnMB2;
extern UVI_UINT8 UserOwnMB3;
extern struct wait_queue * int_wq[7][256];	/* int wait queues (level/vector) */
extern struct wait_queue * dma_wq;

/********************************************************************************
*
*  FUNCTION:   uvi_open
*
*  PURPOSE:    Entry point for open(2) call.
*  PARAMETERS: *inode: (I) Passed in by OS, used for minor number determination
*              *filp:  (I) Unused, passed in by OS
*  RETURNS:    0: Success
*              -ENOMEM:  Unable to map memory
*              -EINVAL
*  GLOBALS:    devname
*
********************************************************************************/

int
uvi_open( struct inode *inode, struct file *filp )
{
    int status;

    static char *me = "uvi_open()";
        
    WHENDEBUG(UVI_DBTRACE) printk(KERN_ERR"%s: Entering %s\n", devname, me);

    atomic_inc( &open_count );

    if( atomic_read( &open_count ) < 2 ) 
    { 
      /* install interrupt service routine */
      status = uviInstallUniverseInterruptHandler();
      if( status != UVI_SUCCESS )
      {
        WHENDEBUG(UVI_DBERROR)
        {
          printk(KERN_ERR"%s: Exiting %s: uviInstallUniverseInterruptHandler() failed," 
          " return status = %d\n", devname, me, status );

        }

        return( -EINVAL );

      }

    }

    WHENDEBUG(UVI_DBTRACE) printk(KERN_ERR"%s: Exiting %s\n", devname, me);
    
    return(0);
    
}   /* End of uvi_open() */


/********************************************************************************
*
*  FUNCTION:   uvi_release
*
*  PURPOSE:    Entry point for close(2) call.
*  PARAMETERS: *inode: (I) Passed in by OS, used for minor number determination
*              *filp:  Unused, passed in by OS
*  RETURNS:    0, -EINVAL 
*  GLOBALS:    devname
*
********************************************************************************/

int uvi_release( struct inode *inode, struct file *filp )
{
    int status;

    static char *me = "uvi_release()";
  
    WHENDEBUG(UVI_DBTRACE) printk(KERN_ERR"%s: Entering %s\n", devname, me);
    
    atomic_dec( &open_count );

    if( atomic_read( &open_count ) < 1 ) 
    { 
      atomic_set( &open_count,0 );

      /* remove interrupt service routine */
      status = uviRemoveUniverseInterruptHandler();
      if( status != UVI_SUCCESS )
      {
        WHENDEBUG(UVI_DBERROR)
        {
          printk(KERN_ERR"%s: Exiting %s: uviRemoveUniverseInterruptHandler() failed," 
               " return status = %d\n", devname, me, status );

        }

        return( -EINVAL );

      }
    }

    WHENDEBUG(UVI_DBTRACE) printk(KERN_ERR"%s: Exiting %s\n", devname, me);
    
    return(0);

}   /* End of uvi_release() */


/********************************************************************************
*
*  FUNCTION:   uvi_mmap
*
*  PURPOSE:    Entry point for mmap(2) call.  Maps VME address into user
*              address space.
*  PARAMETERS: *filp: (I) Passed in by OS, used for minor number determination.
*              *vma: (I) Passed in by OS, used to get virtual and physical 
*                   addresses and size, which are passed into remap_page_range()
*  RETURNS:    0: Success
*              -EAGAIN: Memory mapping failed
*  GLOBALS:    devname
*
********************************************************************************/

int uvi_mmap( struct file *filp, struct vm_area_struct *vma )
{
    static char *me = "uvi_mmap()";
    unsigned long virtaddr;
    unsigned long physaddr;
    unsigned long size;
    
    WHENDEBUG(UVI_DBTRACE) printk(KERN_ERR"%s: Entering %s\n", devname, me);
    
    virtaddr = vma->vm_start;
    physaddr = vma->vm_offset;
    size = vma->vm_end - vma->vm_start;

    WHENDEBUG(UVI_DBMMAP)
    {
        printk(KERN_ERR"%s: %s virtaddr=0x%X physaddr=0x%X size=0x%X\n",
            devname, me, (UVI_UINT32)virtaddr, (UVI_UINT32)physaddr,
            (UVI_UINT32)size );
    }

    if ( physaddr & ~PAGE_MASK )
    {
        WHENDEBUG(UVI_DBERROR)
        {
            printk( KERN_ERR "%s:%s; physical address 0x%x not page aligned.\n", devname,
                    me, (UVI_UINT32)physaddr );
        }
        return -ENXIO;
    }

    if( remap_page_range( virtaddr, physaddr, size, vma->vm_page_prot ) )
    {
        WHENDEBUG(UVI_DBERROR)
        {
            printk(KERN_WARNING"%s: %s's remap_page_range() failed\n", 
                devname, me );
        }
        return( -EAGAIN );
    }

    WHENDEBUG(UVI_DBTRACE) printk(KERN_ERR"%s: Exiting %s\n", devname, me);
    
    return(0);
        
}   /* End of uvi_mmap() */


/********************************************************************************
*
*  FUNCTION:   uvi_ioctl
*
*  PURPOSE:    Entry point for ioctl(2) call.
*  PARAMETERS: *inode: (I) Passed in by OS, used for minor number determination
*              *filp: Passed in by OS, unused.
*              cmd: (I) The ioctl command to be processed
*              arg: (IO) Argument to cmd, or return value from command
*  RETURNS:    0: Success
*              -EINVAL: Invalid parameter or command
*              -EFAULT: Failure during command execution
*              -ETIMEDOUT: Timed out waiting on an interrupt
*  GLOBALS:    devname
*
********************************************************************************/

int
uvi_ioctl( struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg )
{
    static char *me = "uvi_ioctl()";
    struct uviVmeWindowParameters Data;
    struct uviVmeDmaParameters DmaData;
    struct uviInterruptParameters VmeIntData;
    struct uviVmeSlaveWinParameters SlaveWinOpen;
    struct uviVmeSlaveCloseParams SlaveWinClose;
    struct uviMailboxParameters MboxParams;
    int status = 0;
    int dma_status;

    unsigned long dma_width;

    UVI_INT32 mbnum;

    WHENDEBUG(UVI_DBTRACE) printk(KERN_ERR"%s: Entering %s\n", devname, me);

    /* Process the ioctl command */
    switch( cmd )
    {
        case IOCTL_UVI_REQ_VME_INT:
        {
            char *myself = "UVI_REQ_VME_INT";

            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }

            if( copy_from_user( (void *)&VmeIntData, (void *)arg, 
                sizeof(struct uviInterruptParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                    
                return( -EFAULT );
            }

            status = uviRequestUseVmeInt( (int) VmeIntData.IntLevel, (int) VmeIntData.IntVector );

            VmeIntData.IntStatus = status;

            if( status == UVI_SUCCESS )
            { 
              IntTable[VmeIntData.IntLevel - 1][VmeIntData.IntVector].IntrFlags |= USER_OWNS_INT;
            }

            if( copy_to_user( (void *)arg, (void *)&VmeIntData, 
                sizeof(struct uviInterruptParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_to_user() failed\n", 
                        devname, me );
                }
                return( -EFAULT );
            }

            return( UVI_SUCCESS );

            break;
        } /* end case IOCTL_UVI_REQ_VME_INT */            


        case IOCTL_UVI_RELEASE_VME_INT:
        {
            char *myself = "UVI_RELEASE_VME_INT";

            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }

            if( copy_from_user( (void *)&VmeIntData, (void *)arg, 
                sizeof(struct uviInterruptParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                    
                return( -EFAULT );
            }

            /* verify user owns */
            if( IntTable[VmeIntData.IntLevel -1][VmeIntData.IntVector].IntrFlags & USER_OWNS_INT )
            {
              if( IntTable[VmeIntData.IntLevel -1][VmeIntData.IntVector].IntrFlags & USER_READY_INT )
                status = UVI_VMEINT_WAITING;
              else
                status = uviReleaseVmeInt( VmeIntData.IntLevel, VmeIntData.IntVector );
            }
            else
              status = UVI_VMEINT_NOT_USER;

            VmeIntData.IntStatus = status;

            if( copy_to_user( (void *)arg, (void *)&VmeIntData, 
                sizeof(struct uviInterruptParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_to_user() failed\n", 
                        devname, me );
                }
                return( -EFAULT );
            }

            return( UVI_SUCCESS );

            break;
        } /* end case IOCTL_UVI_RELEASE_VME_INT */


        case IOCTL_UVI_CANCEL_VME_INT:
        {
            char *myself = "UVI_CANCEL_VME_INT";

            status = UVI_SUCCESS;

            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }

            if( copy_from_user( (void *)&VmeIntData, (void *)arg, 
                sizeof(struct uviInterruptParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                    
                return( -EFAULT );
            }

            /* verify USER_READY_INT and USER_OWNS_INT are set in IntTable */
            if( IntTable[VmeIntData.IntLevel - 1][VmeIntData.IntVector].IntrFlags & USER_OWNS_INT )
            {
              if( IntTable[VmeIntData.IntLevel - 1][VmeIntData.IntVector].IntrFlags & USER_READY_INT )
              {
                IntTable[VmeIntData.IntLevel - 1][VmeIntData.IntVector].IntrFlags |= CANCEL_USER_OWNS_INT;
                wake_up_interruptible( &int_wq[VmeIntData.IntLevel - 1][VmeIntData.IntVector & 0xFF] );
              }
              else
                status = UVI_VMEINT_NOTWAITING;
            }
            else
              status = UVI_VMEINT_NOT_USER;

            VmeIntData.IntStatus = status;

            if( copy_to_user( (void *)arg, (void *)&VmeIntData, 
                sizeof(struct uviInterruptParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_to_user() failed\n", 
                        devname, me );
                }
                return( -EFAULT );
            }

            return( UVI_SUCCESS );

            break;
        } /* end case IOCTL_UVI_CANCEL_VME_INT */


        case IOCTL_UVI_WAIT_VME_INT:
        {
            char *myself = "UVI_WAIT_VME_INT";

            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }

            if( copy_from_user( (void *)&VmeIntData, (void *)arg, 
                sizeof(struct uviInterruptParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                    
                return( -EFAULT );
            }

            /* verify user owns level and vector */
            if( IntTable[VmeIntData.IntLevel - 1][VmeIntData.IntVector].IntrFlags & USER_OWNS_INT )
            {
              /* uviEnableVmeInt uses interruptible_sleep_on */
              status = uviEnableVmeInt( &VmeIntData );
            }
            else
              status = UVI_VMEINT_NOT_USER;

            WHENDEBUG(UVI_DBINTR)
            {
                printk(KERN_ERR"%s: %s: INT_STATUS: %X STAT: %i\n", 
                    devname, me, VmeIntData.IntStatus, status );
            }

            if( copy_to_user( (void *)arg, (void *)&VmeIntData, 
                sizeof(struct uviInterruptParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_to_user() failed\n", 
                        devname, me );
                }
                return( -EFAULT );
            }

            if( status != UVI_SUCCESS ) 
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: %s: returning: %i\n", 
                        devname, me, status );
                }
                return( status );
            }

            return( UVI_SUCCESS );

            break;
        } /* end case IOCTL_UVI_WAIT_VME_INT */            


        case IOCTL_UVI_REQ_MB:
        {
            char *myself = "UVI_REQ_MB";

            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }

            if( copy_from_user( (void *)&mbnum, (void *)arg, 
                sizeof( UVI_INT32 ) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                return( -EFAULT );
            }

            /* request use of mailbox resource */
            if( uviRequestUseMB( mbnum ) != UVI_SUCCESS ) 
              return( UVI_MBOX_BUSY );

            /* set mailbox controlled by user flag */
            switch( mbnum )
            {
              case 0:
                UserOwnMB0 = UVI_TRUE;
              break;
              case 1:
                UserOwnMB1 = UVI_TRUE;
              break;
              case 2:
                UserOwnMB2 = UVI_TRUE;
              break;
              case 3:
                UserOwnMB3 = UVI_TRUE;
              break;
            }

            return( UVI_SUCCESS );

            break;
        } /* end case IOCTL_UVI_REQ_MB */            

        case IOCTL_UVI_RELEASE_MB:
        {
            char *myself = "UVI_RELEASE_MB";

            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }

            if( copy_from_user( (void *)&mbnum, (void *)arg, 
                sizeof( UVI_INT32 ) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                return( -EFAULT );
            }

            /* release use of mailbox resource */
            if( uviReleaseUseMB( mbnum ) != UVI_SUCCESS ) 
              return( UVI_MBOX_ERROR );

            /* clear mailbox controlled by user flag */
            switch( mbnum )
            {
              case 0:
                UserOwnMB0 = UVI_FALSE;
              break;
              case 1:
                UserOwnMB1 = UVI_FALSE;
              break;
              case 2:
                UserOwnMB2 = UVI_FALSE;
              break;
              case 3:
                UserOwnMB3 = UVI_FALSE;
              break;
            }

            return( UVI_SUCCESS );

            break;
        } /* end case IOCTL_UVI_RELEASE_MB */            

        case IOCTL_UVI_CANCEL_MB:
        {
            char *myself = "UVI_CANCEL_MB";

            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }

            if( copy_from_user( (void *)&mbnum, (void *)arg, 
                sizeof( UVI_INT32 ) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                return( -EFAULT );
            }

            /* cancel wait for mailbox interrupt */
            if( uviCancelMB( mbnum ) != UVI_SUCCESS ) 
              return( UVI_MBOX_ERROR );

            return( UVI_SUCCESS );

            break;
        } /* end case IOCTL_UVI_CANCEL_MB */            

        case IOCTL_UVI_ENABLE_MB:
        {
            char *myself = "UVI_ENABLE_MB";

            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }

            if( copy_from_user( (void *)&MboxParams, (void *)arg, 
                sizeof(struct uviMailboxParameters ) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                return( -EFAULT );
            }

            /* uviEnableMB uses interruptible_sleep_on */
            status = uviEnableMB( MboxParams.MailboxNum,
                                  &MboxParams.MailboxDat,
                                  1 );

            WHENDEBUG(UVI_DBINTR)
            {
                printk(KERN_ERR"%s: %s: MBDAT1: %X STAT: %i\n", 
                    devname, me, MboxParams.MailboxDat, status );
            }

            if( copy_to_user( (void *)arg, (void *)&MboxParams, 
                sizeof(struct uviMailboxParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_to_user() failed\n", 
                        devname, me );
                }
                return( -EFAULT );
            }

            WHENDEBUG(UVI_DBINTR)
            {
                printk(KERN_ERR"%s: %s: MBDAT2: %X STAT: %i\n", 
                    devname, me, MboxParams.MailboxDat, status );
            }

            if( status != UVI_SUCCESS ) 
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: %s: returning: %i\n", 
                        devname, me, status );
                }
                return( status );
            }

            return( UVI_SUCCESS );

            break;
        } /* end case IOCTL_UVI_ENABLE_MB */            

        /* added 8/18/00 sjl */
        case IOCTL_UVI_SLAVE_CLOSE:
        {
            char *myself = "SLAVE WIN CLOSE";
            
            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }
            
            if( copy_from_user( (void *)&SlaveWinClose, (void *)arg, 
                sizeof(struct uviVmeSlaveCloseParams) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                    
                return( -EFAULT );
            }

            status = uviCloseSlaveWin( &SlaveWinClose );

            if( status != UVI_SUCCESS )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: uviConfigSlaveWin() failed,"
                        " return status = %d\n", devname, me, status );
                }
                
                return( -EINVAL );
            }
        }
        break;

        /* added 8/17/00 sjl */
        case IOCTL_UVI_SLAVE_OPEN:
        {
            char *myself = "SLAVE WIN OPEN";
            
            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }
            
            if( copy_from_user( (void *)&SlaveWinOpen, (void *)arg, 
                sizeof(struct uviVmeSlaveWinParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                    
                return( -EFAULT );
            }

            status = uviConfigSlaveWin( &SlaveWinOpen );

            if( status != UVI_SUCCESS )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: uviConfigSlaveWin() failed,"
                        " return status = %d\n", devname, me, status );
                }
                
                return( -EINVAL );
            }
        }
        break;

        /* added 9/11/00 sjl */
        case IOCTL_UVI_REQ_DMA:
        {
            char *myself = "UVI_REQ_DMA";

            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }

            /* request use of dma resource */
            if( uviRequestUseDma( ) != UVI_SUCCESS ) return( UVI_DMA_BUSY );

            /* set DMA controlled by user flag */
            UserOwnDma |= USER_OWNS_DMA;

            break;
        } /* end case IOCTL_UVI_REQ_DMA */            

        /* added 9/11/00 sjl */
        case IOCTL_UVI_RELEASE_DMA:
        {
            char *myself = "UVI_RELEASE_DMA";

            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }

            /* clear DMA controlled by user flag */
            UserOwnDma = UVI_FALSE;

            uviReleaseUseDma( );

            break;
        } /* end case IOCTL_UVI_RELEASE_DMA */            

        case IOCTL_UVI_CANCEL_DMA_WAIT:
        {
            char *myself = "UVI_CANCEL_DMA_WAIT";

            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }

            UserOwnDma |= CANCEL_USER_WAIT_DMA;
            wake_up_interruptible( &dma_wq ); 

            break;
        } /* end case IOCTL_UVI_CANCEL_DMA_WAIT */            

        /* added 7/25/00 sjl */
        case IOCTL_UVI_DMAIO:
        {
            char *myself = "DMAIO";
            
            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }
            
            if( copy_from_user( (void *)&DmaData, (void *)arg, 
                sizeof(struct uviVmeDmaParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                    
                return( -EFAULT );
            }

            switch( DmaData.DataWidth )
            {
              case 8:
                dma_width = V_D8;
              break;
              
              case 16:
                dma_width = V_D16;
              break;
              
              case 32:
                dma_width = V_D32;
              break;
              
              case 64:
                dma_width = V_D64;
              break;

              default: return -EINVAL;
            }

             dma_status = uviAccessDmaBuffer(DMA_BUFF_BASE + DmaData.DmaOffset,              
                                             DmaData.VmeBusAddr,           
                                             DmaData.DmaSize,           
                                             DmaData.AddrMod,                 
                                             dma_width,                       
                                             DmaData.DmaRdWr,
                                             &DmaData.DmaCompStat,
                                             UVI_USER );

            if( copy_to_user( (void *)arg, (void *)&DmaData, 
                sizeof(struct uviVmeDmaParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_to_user() failed\n", 
                        devname, me );
                }
                    
                return( -EFAULT );
            }

            if( dma_status != UVI_SUCCESS )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: uviAccessDmaBuffer() failed,"
                        " return status = %d\n", devname, me, status );
                }
                
                return( dma_status );
            }

        }
        break;

        case IOCTL_UVI_GET_WINDOW:
        {
            char *myself = "GET_WINDOW";
            
            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }
            
            if( copy_from_user( (void *)&Data, (void *)arg, 
                sizeof(struct uviVmeWindowParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                    
                return( -EFAULT );
            }

            status = uviGetVmeWindow(&Data);
            if( status != UVI_SUCCESS )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: uviGetVmeWindow() failed,"
                        " return status = %d\n", devname, me, status );
                }
                
                return( -EINVAL );
                
            }

            if( copy_to_user( (void *)arg, (void *)&Data, 
                sizeof(struct uviVmeWindowParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_to_user() failed\n", 
                        devname, me );
                }

                return( -EFAULT );
            }
        }
        break;


        case IOCTL_UVI_CLOSE_WINDOW:
        {
            char *myself = "CLOSE_WINDOW";
            
            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }
            
            if( copy_from_user( (void *)&Data, (void *)arg, 
                sizeof(struct uviVmeWindowParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                    
                return( -EFAULT );
            }

            status = uviReleaseVmeWindow(&Data);
            if( status != UVI_SUCCESS )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: uviReleaseVmeWindow() failed,"
                        " return status = %d\n", devname, me, status );
                }
                
                return( -EINVAL );
                
            }

            if( copy_to_user( (void *)arg, (void *)&Data, 
                sizeof(struct uviVmeWindowParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_to_user() failed\n", 
                        devname, me );
                }

                return( -EFAULT );
            }
        }
        break;


        case IOCTL_UVI_SET_DEBUG_FLAGS:
        {
            char *myself = "SET_DEBUG_FLAGS";

            UviDebugFlags = (UVI_UINT32) arg;
                        
            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s, flags = 0x%08X\n", 
                    devname, me, myself, UviDebugFlags );
            }
        }
        break;


        case IOCTL_UVI_GET_DEBUG_FLAGS:
        {
            char *myself = "GET_DEBUG_FLAGS";
            
            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s, flags = 0x%08X\n", 
                    devname, me, myself, UviDebugFlags );
            }

            if( copy_to_user( (void *)arg, (void *)&UviDebugFlags, 
                sizeof(UVI_UINT32) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_to_user() failed\n",
                        devname, me );
                }
                
                return( -EFAULT );
            }            
            
        }
        break;


        case IOCTL_UVI_PROBE_ADDRESS:
        {
            char *myself = "PROBE_ADDRESS";
            int  width;
            
            WHENDEBUG(UVI_DBIOCTL)
            {
                printk(KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }
            
            if( copy_from_user( (void *)&Data, (void *)arg, 
                sizeof(struct uviVmeWindowParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                    
                return( -EFAULT );
            }
            
            width = Data.ReqWndSize;
            Data.ReqWndSize = 1;
            Data.SysModeFlag = UVI_KERNEL;
            switch( uviProbeAddress(&Data, width) )
            {
                case UVI_SUCCESS:
                    WHENDEBUG(UVI_DBIOCTL)
                    {
                        printk(KERN_ERR"%s: %s, %s  Successful probe of VME "
                            "address 0x%08X, addrmod = 0x%02X\n",
                            devname, me, myself, Data.VmeBusAddr, Data.AddrMod );
                    }
                    break;
                    
                case UVI_BERR:
                    WHENDEBUG(UVI_DBIOCTL)
                    {
                        printk(KERN_ERR"%s: %s, %s  Bus error at probe of VME "
                            "address 0x%08X, addrmod = 0x%02X\n",
                            devname, me, myself, Data.VmeBusAddr, Data.AddrMod );
                    }
                    
                    return( -EIO );
                    break;
                    
                default:
                    return( -EINVAL );
            }
            
        }
        break;


        case IOCTL_UVI_ENABLE_EVENT:
        {
            char *myself = "ENABLE_EVENT";
           
            WHENDEBUG( UVI_DBIOCTL )
            {
                printk( KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }
/*            
            if( copy_from_user( (void *)&Data, (void *)arg, 
                sizeof(struct uviVmeWindowParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                    
                return( -EFAULT );
            }
*/            
        }
        break;


        case IOCTL_UVI_RECEIVE_EVENT:
        {
            char *myself = "RECEIVE_EVENT";
           
            WHENDEBUG( UVI_DBIOCTL )
            {
                printk( KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }
/*            
            if( copy_from_user( (void *)&Data, (void *)arg, 
                sizeof(struct uviVmeWindowParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                    
                return( -EFAULT );
            }
*/            
        }
        break;


        case IOCTL_UVI_DISABLE_EVENT:
        {
            char *myself = "DISABLE_EVENT";
           
            WHENDEBUG( UVI_DBIOCTL )
            {
                printk( KERN_ERR"%s: %s cmd = %s\n", devname, me, myself );
            }
/*            
            if( copy_from_user( (void *)&Data, (void *)arg, 
                sizeof(struct uviVmeWindowParameters) ) > 0 )
            {
                WHENDEBUG(UVI_DBERROR)
                {
                    printk(KERN_ERR"%s: Exiting %s: copy_from_user() failed\n", 
                        devname, me );
                }
                    
                return( -EFAULT );
            }
*/            
        }
        break;


        default:  
            WHENDEBUG( UVI_DBERROR | UVI_DBIOCTL )
            {
                printk( KERN_ERR"%s: Exiting %s:Invalid ioctl command\n", 
                    devname, me );
            }
            
            printk(KERN_ERR"\ndriver_entry - IOCTL default found\n");
            return( -EINVAL );
            break;        
    }
    

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_ERR"%s: Exiting %s\n", devname, me );
    
    return( 0 );
    
}   /* End of uvi_ioctl() */


