/*
===============================================================================
                            COPYRIGHT NOTICE

    Copyright (C) 2000 VMIC
    International Copyright Secured.  All Rights Reserved.

-------------------------------------------------------------------------------
    Filename: proc_page.c  Version: 1.1  Modification Date: 00/03/21 10:15:02
-------------------------------------------------------------------------------
    Description: Linux Proc Page routine for the Universe driver
-------------------------------------------------------------------------------

    Revision History:
    Date         Who  Ver   Reason For Change
    -----------  ---  ----  ---------------------------------------------------
    21-MAR-2000  AE   1.1   Created.

===============================================================================
*/

#ident "@(#)proc_page.c 1.1 00/06/27 VMIC"


#include "uvi_module.h"

/******************************************************************************
*
*  FUNCTION:   uviReadProcPage
*
*  PURPOSE:    Provides contents of the /proc/uvi file.
*  PARAMETERS: *buf:    (IO) Character buffer, passed in by OS
*              **start: Unused, passed in by OS
*              offset:  Unused, passed in by OS
*              len:     Unused, passed in by OS
*              unused:  Unused, passed in by OS
*  RETURNS:    Number of bytes read
*  GLOBALS:    MODULE_NAME, uviDeviceInfo
*
******************************************************************************/

int
uviReadProcPage( char *buf, char **start, off_t offset, int len, int unused )
{
    static char *ThisFunction = "uviReadProcPage()";
    int bytesRead = 0;        /* Returned length                        */

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO"%s: Entering %s\n", MODULE_NAME,
                                     ThisFunction );
    
    bytesRead = sprintf( buf, 
        "\nWelcome to the VMIVME Universe Proc Page!\n\n" );

    bytesRead += sprintf( buf+bytesRead, "Current Debug Flags = 0x%04X\n\n",
                          UviDebugFlags );

    bytesRead += sprintf( buf+bytesRead, "Information from uviInitDev:\n\n" );

    if ( PlxFound )
    {
        bytesRead += sprintf( buf+bytesRead, "PLX interface found. Vendor id is %x,"
                              " device id is %x, dev function is %x.\n",
                              UviPlxDevice->vendor,
                              UviPlxDevice->device,                          
                              UviPlxDevice->devfn );    
        bytesRead += sprintf( buf+bytesRead, "Class is %x, hdr_type is %x.\n",
                              UviPlxDevice->class, UviPlxDevice->hdr_type );
        bytesRead += sprintf( buf+bytesRead, "Master capable value is %d, irq is %d.\n",
                              UviPlxDevice->master, UviPlxDevice->irq );
        bytesRead += sprintf( buf+bytesRead, "First base addr is %lx, rom addr is %lx.\n",
                              UviPlxDevice->base_address[0], UviPlxDevice->rom_address );
    }
    else
    {
        bytesRead += sprintf( buf+bytesRead, "ISA style registers in use.\n" );
        bytesRead += sprintf( buf+bytesRead, "VMIC cpu board ID reg val is 0x%04x.\n",
                              IsaIdRegVal );
        bytesRead += sprintf( buf+bytesRead, "VMIC cpu board COMM reg val is 0x%04x.\n",
                              IsaCmdRegVal );
    }

    bytesRead += sprintf( buf+bytesRead, "\nUniverse interface found. Vendor id is %x,"
                          " device id is %x, dev function is %x.\n",
                          UviDevice->vendor,
                          UviDevice->device,
                          UviDevice->devfn );
    bytesRead += sprintf( buf+bytesRead, "Class is %x, hdr_type is %x.\n",
                          UviDevice->class, UviDevice->hdr_type );
    bytesRead += sprintf( buf+bytesRead, "Master capable value is %d, irq is %d.\n",
                          UviDevice->master, UviDevice->irq );
    bytesRead += sprintf( buf+bytesRead, "First base addr is %lx, rom addr is %lx.\n",
                          UviDevice->base_address[0], UviDevice->rom_address );

     WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO"%s: Exiting %s\n", MODULE_NAME,
                                     ThisFunction );
    
    return( bytesRead );

}   /* End of uviReadProcPage() */



/*****************************************************************************/

/* Define the proc page support structure */
struct proc_dir_entry uviProcEntry = 
{
    0,                      /* Inode, set by kernel         */
    DEVNAME_SIZE,           /* Size of name                 */
    "uvi",                  /* Name of page                 */
    S_IFREG | S_IRUGO,      /* Mode                         */
    1, 0, 0,                /* Nlinks, owner, group         */
    0,                      /* Size (not used)              */
    NULL,                   /* Default operations           */
    uviReadProcPage,        /* Our read function            */
    
    /* The rest defaults to NULL */
};

