/*
=================================================================================
                            COPYRIGHT NOTICE

    Copyright (C) 2000 VMIC
    International Copyright Secured.  All Rights Reserved.

---------------------------------------------------------------------------------
    Filename: uvimod_init.c  Version: 1.1  Modification Date: 00/03/21 10:15:03
---------------------------------------------------------------------------------
    Description: Universe module initialization upon loading.
---------------------------------------------------------------------------------

    Revision History:
    Date         Who  Ver   Reason For Change
    -----------  ---  ----  -----------------------------------------------------
    21-MAR-2000  AE   1.1   Created.

=================================================================================
*/
#ident "@(#)uvimod_init.c 1.1 00/06/27 VMIC"


#define ALLOC_DATA
#include "uvi_module.h"


/********************************************************************************
*
*  FUNCTION:   init_module
*
*  PURPOSE:    Standard entry point to handle registration of the UVIMOD
*              driver module.
*
*  PARAMETERS: void
*
*  RETURNS:    UVI_SUCCESS    : Success
*              -ENXIO         : No such device or address
*              UVI_NO_PCI     : PCI bus not present
*              UVI_PCI_NOTCFG : PCI support not configured for OS
*
*  GLOBALS:    UviDebugFlags, IrqOpSpinLock, IntTable, major_num, devname
*
********************************************************************************/
UVI_INT32 init_module( void )
{

    static char *ThisFunction = "init_module";

    UVI_INT32 UviInitDevStatus,
              result; /* added by J.W. */

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_ERR "%s:%s; Entering ...\n", MODULE_NAME,
                                    ThisFunction );

 /* Default debug flags. */
    UviDebugFlags = UVI_DBERROR;

    WHENDEBUG( UVI_DBINIT )
    {
        printk( KERN_ERR "%s:%s; Initializing the universe module.\n",
                MODULE_NAME, ThisFunction );
    }

/*
** Make sure the 'get window' and 'irq ops' spin locks are initially unlocked.  It
** turns out that the window ops spinlocks are not needed for a UP system, but may
** be needed later for an SMP system.
*/
/*    WndOp_SpinLock = SPIN_LOCK_UNLOCKED; */
    IrqOpSpinLock = SPIN_LOCK_UNLOCKED;

/* Zero out IRQ table. */
    memset( IntTable, 0, sizeof( IntTable ) );

/* Is PCI support configured?  If not, exit! */
    #ifdef CONFIG_PCI
        WHENDEBUG( UVI_DBINIT )
        {
            printk( KERN_ERR "%s:%s; PCI configured in OS ...\n", MODULE_NAME,
                    ThisFunction );
        }
/* If PCI support configured, is a PCI bus available?  If not, exit! */
        if ( pci_present() )
        {
            WHENDEBUG( UVI_DBINIT )
            {
                printk( KERN_ERR "%s:%s; PCI bus is available.\n", MODULE_NAME,
                        ThisFunction );
            }

            UviInitDevStatus = uviInitDev();
            if ( UviInitDevStatus != UVI_SUCCESS )
            {
                WHENDEBUG( UVI_DBERROR )
                {
                    printk( KERN_ERR "%s:%s; Failed to initialize the universe device,"
                            " abort!\n", MODULE_NAME, ThisFunction );
                }
                return( -ENXIO );
            }

            major_num = 0;
            result = register_chrdev( major_num, devname, &uvi_fops );
            if( result < 0 )
            {
                return( result );
            }
    
/* We succeeded, so store our new major device number */
            if( major_num == UVI_DYNAMIC_MAJOR_NUMBER )
            {
                major_num = result;
            }
    
/* Register our /proc/uvimodule page */
            proc_register( &proc_root, &uviProcEntry );
        }
        else
        {
            WHENDEBUG( UVI_DBERROR )
            {
                printk( KERN_ERR "%s:%s; you do NOT have a PCI based system, abort!\n",
                        MODULE_NAME, ThisFunction );
            }
            return( UVI_NO_PCI );
        }
    #else
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; PCI is not configured for this OS!\n",
                    MODULE_NAME, ThisFunction );
        }
        return( UVI_PCI_NOTCFG );
    #endif

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_ERR "%s:%s; Exiting ...\n", MODULE_NAME,
                                    ThisFunction );

    return( UVI_SUCCESS );

} /* end 'init_module */


/********************************************************************************
*
*  FUNCTION:   cleanup_module
*
*  PURPOSE:    Standard entry point to handle unregistration of the UVIMOD
*              driver module
*  PARAMETERS: void
*  RETURNS:    void
*  GLOBALS:    UnivBaseAddr, Univ1Found, PlxFound, PlxMappedAddr, IsaCmdRegVal,
*              major_num, devname
*
********************************************************************************/
void cleanup_module( void )
{

    static char *ThisFunction = "cleanup_module";

    UVI_UINT16 IsaCommandBits = 0x03FF;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_ERR "%s:%s; Entering ...\n", MODULE_NAME,
                                    ThisFunction );
 
/* Disable all universe interrupts master/slave windows and VME 
** register access
*/
    if ( Univ1Found )
    {
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= LINT_DIS_ALL_U1;
        *(UVI_UINT32 *)( UnivBaseAddr + VINT_EN ) &= VINT_DIS_ALL_U1;
        *(UVI_UINT32 *)( UnivBaseAddr + SLSI ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + LSI0_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + LSI1_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + LSI2_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + LSI3_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + VSI0_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + VSI1_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + VSI2_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + VSI3_CTL ) = 0; 
    }
    else
    {
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= LINT_DIS_ALL_U2;
        *(UVI_UINT32 *)( UnivBaseAddr + VINT_EN ) &= VINT_DIS_ALL_U2;
        *(UVI_UINT32 *)( UnivBaseAddr + SLSI ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + LSI0_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + LSI1_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + LSI2_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + LSI3_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + LSI4_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + LSI5_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + LSI6_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + LSI7_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + VSI0_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + VSI1_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + VSI2_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + VSI3_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + VSI4_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + VSI5_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + VSI6_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + VSI7_CTL ) = 0; 
        *(UVI_UINT32 *)( UnivBaseAddr + VRAI_CTL ) = 0; 
    }

/* Disable the vme bus upon leaving.  Use the PLX device if found. */
    if ( PlxFound )
    {  
        WHENDEBUG( UVI_DBEXTRA )
        {
            printk( KERN_ERR "%s:%s; Disabling the vme bus.\n", MODULE_NAME,
                    ThisFunction );
        }

        WHENDEBUG( UVI_DBEXTRA )
        {
            printk( KERN_ERR "%s:%s; PLX data read is 0x%08x.\n", MODULE_NAME,
                    ThisFunction, *(UVI_UINT16 *)PlxMappedAddr );
        }

        *(UVI_UINT16 *)PlxMappedAddr &= VME_DIS_PLX;

        WHENDEBUG( UVI_DBEXTRA )
        {
          printk( KERN_ERR "%s:%s; New PLX data val read is 0x%08x.\n", MODULE_NAME,
                  ThisFunction, *(UVI_UINT16 *)PlxMappedAddr );
        }
    }
    else
/* Try using the COMM reg. */
    {
        IsaCmdRegVal = readw( NOTPLX_CMDREG_ADDR );
        WHENDEBUG( UVI_DBEXTRA )
        {
            printk( KERN_ERR "%s:%s; ISA command reg initial value is 0x%08x.\n",
                    MODULE_NAME, ThisFunction, IsaCmdRegVal );
        }

        IsaCmdRegVal &= IsaCommandBits;
        writew( IsaCmdRegVal, NOTPLX_CMDREG_ADDR );

        IsaCmdRegVal = readw( NOTPLX_CMDREG_ADDR );
        WHENDEBUG( UVI_DBEXTRA )
        {
            printk( KERN_ERR "%s:%s; ISA command reg value is now 0x%08x.\n",
                    MODULE_NAME, ThisFunction, IsaCmdRegVal );
        }
    }

    iounmap( (unsigned char *)PlxMappedAddr );
    PlxMappedAddr = 0x0;

    iounmap( (unsigned char *)UnivBaseAddr );
    UnivBaseAddr = 0x0;

    /* Release our major device number */
    unregister_chrdev( major_num, devname );

    /* Release our /proc/uvimodule page */
    proc_unregister( &proc_root, uviProcEntry.low_ino );

	/** End of new code ******************************************************/

    WHENDEBUG( UVI_DBTRACE )
    {
         printk( KERN_ERR "%s:%s; Exiting ...\n", MODULE_NAME,
                                    ThisFunction );
    }

    return;

} /* end 'cleanup_module' */





























