/*
=================================================================================
                            COPYRIGHT NOTICE

    Copyright (C) 2000 VMIC
    International Copyright Secured.  All Rights Reserved.

---------------------------------------------------------------------------------
    Filename: uvimod_ops.c  Version: 1.1  Modification Date: 00/03/21 10:15:03
---------------------------------------------------------------------------------
    Description: Setup VME windows, read/write operations
---------------------------------------------------------------------------------

    Revision History:
    Date         Who  Ver   Reason For Change
    -----------  ---  ----  -----------------------------------------------------
    21-MAR-2000  AE   1.1   Created.


=================================================================================
*/
#ident "@(#)uvimod_ops.c 1.1 00/06/27 VMIC"


#define EXPORT_SYMTAB
#define LOCAL_PG_MSK    0xFFFFF000

#include <linux/module.h>

#include "uvi_module.h"
#include "uvi_universe.h"

EXPORT_SYMBOL_NOVERS( uviInitVme );
EXPORT_SYMBOL_NOVERS( uviGetVmeWindow );
EXPORT_SYMBOL_NOVERS( uviReleaseVmeWindow );
EXPORT_SYMBOL_NOVERS( uviProbeAddress );
EXPORT_SYMBOL_NOVERS( uviGetUniverseInterruptLevel );
EXPORT_SYMBOL_NOVERS( uviEnableInterrupt );
EXPORT_SYMBOL_NOVERS( uviCheckIntSource );
EXPORT_SYMBOL_NOVERS( uviReArmInterrupt );
EXPORT_SYMBOL_NOVERS( uviDisableInterrupt );
EXPORT_SYMBOL_NOVERS( uviEnableDma );
EXPORT_SYMBOL_NOVERS( uviDisableDma );
EXPORT_SYMBOL_NOVERS( uviAccessDmaBuffer );
EXPORT_SYMBOL_NOVERS( uviRequestUseDma );
EXPORT_SYMBOL_NOVERS( uviReleaseUseDma );
EXPORT_SYMBOL_NOVERS( uviEnableDmaInterrupt );
EXPORT_SYMBOL_NOVERS( uviDisableDmaInterrupt );
EXPORT_SYMBOL_NOVERS( uviRequestUseMB );
EXPORT_SYMBOL_NOVERS( uviReleaseUseMB );
EXPORT_SYMBOL_NOVERS( uviCancelMB );
EXPORT_SYMBOL_NOVERS( uviRequestUseVmeInt );
EXPORT_SYMBOL_NOVERS( uviEnableVmeInt );

/* 'global' only inside this file */
static UVI_UINT8 LvlInUseCnt[7] = {0,0,0,0,0,0,0};

/* LvlIndex, */
static UVI_INT8 UnivIntrInUse = 0;

static UVI_UINT8 LintEnDmaEn,
                 VintEnDmaEn;

static UVI_INT32 FuncStatus;

static UVI_UINT32 Flags, /* irq spinlock flags */
                  UnivInterruptId;

/* global slave window owner/busy flag */
static UVI_UINT32 SlvWindowBusy[8] = {0,0,0,0,0,0,0,0};

struct wait_queue * dma_wq = NULL;	/* dma wait queue */
struct wait_queue * mb0_wq = NULL;	/* MB0 wait queue */
struct wait_queue * mb1_wq = NULL;	/* MB1 wait queue */
struct wait_queue * mb2_wq = NULL;	/* MB2 wait queue */
struct wait_queue * mb3_wq = NULL;	/* MB3 wait queue */
struct wait_queue * int_wq[7][256];	/* int wait queues (level/vector) */

atomic_t dma_lock = ATOMIC_INIT(0);
atomic_t MB0_lock = ATOMIC_INIT(0);
atomic_t MB1_lock = ATOMIC_INIT(0);
atomic_t MB2_lock = ATOMIC_INIT(0);
atomic_t MB3_lock = ATOMIC_INIT(0);

static UVI_UINT8 UserOwnDma = 0;        /* also used in driver_entry */
static UVI_UINT8 UserOwnMB0 = 0;
static UVI_UINT8 UserOwnMB1 = 0;
static UVI_UINT8 UserOwnMB2 = 0;
static UVI_UINT8 UserOwnMB3 = 0;

/********************************************************************************
*
*  FUNCTION:   uviInitVme
*
*  PURPOSE:    To check that the universe device has been setup, etc.
*
*  PARAMETERS: None 
*
*  RETURNS:    UVI_SUCCESS : Success
*              UVI_NO_UDEV : The universe device missing or not properly initialized.
*
*  GLOBALS:    UdeviceOk
*
********************************************************************************/
UVI_INT32 uviInitVme( void )
{

    static char *ThisFunction = "uviInitVme";

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

    if ( !UdeviceOk )
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; The universe functionality is not available!\n",
                    MODULE_NAME, ThisFunction );
        }
        return( UVI_NO_UDEV );
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    return( UVI_SUCCESS );

} /* end of 'uviInitVme' */


/********************************************************************************
*
*  FUNCTION:   uviGetVmeWindow
*
*  PURPOSE:    To return a pointer to a universe VME 'window'.  These 'windows'
*              have been defined/set up already, we just give the user access to
*              a requested 'window', if the request is valid.
*
*  PARAMETERS: Pointer to VME window parameters structure, VmeWindowParams.
*
*  RETURNS:    UVI_SUCCESS       : Success
*              UVI_WNDSPEC_ERROR : VME 'window' specification error
*              UVI_INV_ADDRMOD   : Invalid/unsupported address modifier
*              MODUVI_NULL_PTR   : pointer not obtained
*
*  GLOBALS:    FuncStatus, UnivBaseAddr
*
********************************************************************************/
UVI_INT32 uviGetVmeWindow( struct uviVmeWindowParameters *VmeWindowParams )
{

    static char *ThisFunction = "uviGetVmeWindow";

    unsigned char *PtrToRequestedWindow = 0; /* pointer to 'soft device' window */

    UVI_UINT32 LocalVmeBaseAddr,
               LocalVmeTopAddr,
               ImageBaseAddr = 0,
               ReqVmeAddr;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

    FuncStatus = UVI_SUCCESS;

/* Lock function while in use. */
/*    spin_lock( &WndOp_SpinLock ); */

    WHENDEBUG( UVI_DBEXTRA )
    {
        printk( KERN_INFO "%s:%s; Req bus address is 0x%x.\n", MODULE_NAME, ThisFunction,
                VmeWindowParams->VmeBusAddr );
        printk( KERN_INFO "%s:%s; Req window size is 0x%x.\n", MODULE_NAME, ThisFunction,
                VmeWindowParams->ReqWndSize );
        printk( KERN_INFO "%s:%s; Req addr modifier is 0x%x.\n", MODULE_NAME, ThisFunction,
                VmeWindowParams->AddrMod );
        printk( KERN_INFO "%s:%s; Req system mode is 0x%x.\n", MODULE_NAME, ThisFunction,
                VmeWindowParams->SysModeFlag );
    }

/*
** Is this request valid in terms of address modifier, window size, and window
** bounds.
*/
    switch ( VmeWindowParams->AddrMod )
    { 
        case V_A16UD:
        case V_A16SD:
            if ( VmeWindowParams->ReqWndSize < 1 ||
                      VmeWindowParams->ReqWndSize > MaxA16UDSpace ||
                                 VmeWindowParams->ReqWndSize > MaxA16SDSpace  )
            {
                WHENDEBUG( UVI_DBERROR )
                {
                    printk( KERN_ERR "%s:%s; Requested window size out of A16 range.\n",
                            MODULE_NAME, ThisFunction );
                }
                FuncStatus = UVI_WNDSPEC_ERROR;
            }
            if ( ( VmeWindowParams->VmeBusAddr < 0 || ( VmeWindowParams->VmeBusAddr +
                    VmeWindowParams->ReqWndSize ) > V_A16_BOUND ) )
            {
                WHENDEBUG( UVI_DBERROR )
                {
                    printk( KERN_ERR "%s:%s; Requested bus address outside A16 window.\n",
                            MODULE_NAME, ThisFunction );
                }
                FuncStatus = UVI_WNDSPEC_ERROR;
            }
            break;

        case V_A24UD:
        case V_A24SD:
            if ( VmeWindowParams->ReqWndSize < 1 ||
                    VmeWindowParams->ReqWndSize > MaxA24UDSpace ||
                                    VmeWindowParams->ReqWndSize > MaxA24SDSpace )
            {
                WHENDEBUG( UVI_DBERROR )
                {
                    printk( KERN_ERR "%s:%s; Requested window size out of A24 range.\n",
                            MODULE_NAME, ThisFunction );
                }
                FuncStatus = UVI_WNDSPEC_ERROR;
            }
            if ( ( VmeWindowParams->VmeBusAddr < 0 || ( VmeWindowParams->VmeBusAddr +
                    VmeWindowParams->ReqWndSize ) > V_A24_BOUND ) )
            {
                WHENDEBUG( UVI_DBERROR )
                {
                    printk( KERN_ERR "%s:%s; Requested bus address outside A24 window.\n",
                            MODULE_NAME, ThisFunction );
                }
                FuncStatus = UVI_WNDSPEC_ERROR;
            }
            break;

        case V_A32UD:
            if ( VmeWindowParams->ReqWndSize < 1 ||
                      VmeWindowParams->ReqWndSize > MaxA32UDSpace )
            {
                WHENDEBUG( UVI_DBERROR )
                {
                    printk( KERN_ERR "%s:%s; Requested window size out of A32UD range.\n",
                            MODULE_NAME, ThisFunction );
                }
                FuncStatus = UVI_WNDSPEC_ERROR;
            }

            LocalVmeBaseAddr = *(UVI_UINT32 *)( UnivBaseAddr + LSI2_BS ) +
                                              *(UVI_UINT32 *)( UnivBaseAddr + LSI2_TO );

            LocalVmeTopAddr = LocalVmeBaseAddr + ( *(UVI_UINT32 *)( UnivBaseAddr + LSI2_BD ) -
                                              *(UVI_UINT32 *)( UnivBaseAddr + LSI2_BS ) );

            if ( ( VmeWindowParams->VmeBusAddr < LocalVmeBaseAddr ) ||
                  ( ( VmeWindowParams->VmeBusAddr + VmeWindowParams->ReqWndSize ) >
                                                                      LocalVmeTopAddr ) )
            {
                WHENDEBUG( UVI_DBERROR )
                {
                    printk( KERN_ERR "%s:%s; Requested bus address outside A32UD window.\n",
                            MODULE_NAME, ThisFunction );
                    FuncStatus = UVI_WNDSPEC_ERROR;
                }
            }
            break;

        case V_A32SD:
            if ( VmeWindowParams->ReqWndSize < 1 ||
                      VmeWindowParams->ReqWndSize > MaxA32SDSpace )
            {
                WHENDEBUG( UVI_DBERROR )
                {
                    printk( KERN_ERR "%s:%s; Requested window size out of A32SD range.\n",
                            MODULE_NAME, ThisFunction );
                }
                FuncStatus = UVI_WNDSPEC_ERROR;
            }

            LocalVmeBaseAddr = *(UVI_UINT32 *)( UnivBaseAddr + LSI3_BS ) +
                                              *(UVI_UINT32 *)( UnivBaseAddr + LSI3_TO );

            LocalVmeTopAddr = LocalVmeBaseAddr + ( *(UVI_UINT32 *)( UnivBaseAddr + LSI3_BD ) -
                                              *(UVI_UINT32 *)( UnivBaseAddr + LSI3_BS ) );

            if ( ( VmeWindowParams->VmeBusAddr < LocalVmeBaseAddr ) ||
                  ( ( VmeWindowParams->VmeBusAddr + VmeWindowParams->ReqWndSize ) >
                                                                      LocalVmeTopAddr ) )
            {
                WHENDEBUG( UVI_DBERROR )
                {
                    printk( KERN_ERR "%s:%s; Requested bus address outside A32SD window.\n",
                            MODULE_NAME, ThisFunction );
                    FuncStatus = UVI_WNDSPEC_ERROR;
                }
            }
            break;

        default:
            WHENDEBUG( UVI_DBERROR )
            {
                printk( KERN_ERR "%s:%s; Address modifier not supported. 0x%x.\n",
                        MODULE_NAME, ThisFunction, VmeWindowParams->AddrMod );
            }
            FuncStatus = UVI_INV_ADDRMOD;
            break;
    } /* end 'switch' on access mode */

    if ( FuncStatus != UVI_SUCCESS )
    {
        goto ReturnStatus;
    }

/* Get the image base address. */
    switch( VmeWindowParams->AddrMod )
    {
        case V_A16UD:
            ImageBaseAddr = SPECWND_A16UD_BASE;
            break;
        case V_A16SD:
            ImageBaseAddr = SPECWND_A16SD_BASE;
            break;
        case V_A24UD:
            ImageBaseAddr = *(UVI_UINT32 *)( UnivBaseAddr + LSI0_BS );
            break;
        case V_A24SD:
            ImageBaseAddr = *(UVI_UINT32 *)( UnivBaseAddr + LSI1_BS );
            break;
        case V_A32UD:
            ImageBaseAddr = *(UVI_UINT32 *)( UnivBaseAddr + LSI2_BS );
            break;
        case V_A32SD:
            ImageBaseAddr = *(UVI_UINT32 *)( UnivBaseAddr + LSI3_BS );
            break;

        default: /* valid access mode already tested */
            break;
    } /* end 'switch' on access mode */

    if ( ImageBaseAddr == (UVI_UINT32)NULL )
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; Null image base address value returned.\n",
                    MODULE_NAME, ThisFunction );
        }
        FuncStatus = UVI_NULL_PTR;
        goto ReturnStatus;
    }

/*
** Get a ptr to the requested window and place in the module window parameters
** structure.  If the request comes from the user level, then place the window
** base address + the requested bus address rounded down to the nearest page
** boundary, in the 'PhysAddr' variable.
** This value can be 'mmapped' to get a 'user' pointer that the vme bus address
** can be added to, for device access.
**
** Due to the large address range available for A32 space, some manipulation of
** the translation offset register by the user is possible.  If this is the case
** then we must calculate an address offset using the user defined A32UD or SD
** base values.  We do the same for A16 and A24 space to maintain code consistency,
** even though the A16/A24 base values stay at 0.
*/
    ReqVmeAddr = 0;
    switch( VmeWindowParams->AddrMod )
    {
        case V_A16UD:
            ReqVmeAddr = VmeWindowParams->VmeBusAddr - VME_A16UD_BASE;
            break;
        case V_A16SD:
            ReqVmeAddr = VmeWindowParams->VmeBusAddr - VME_A16SD_BASE;
            break;
        case V_A24UD:
            ReqVmeAddr = VmeWindowParams->VmeBusAddr - VME_A24UD_BASE;
            break;
        case V_A24SD:
            ReqVmeAddr = VmeWindowParams->VmeBusAddr - VME_A24SD_BASE;
            break;
        case V_A32UD:
            ReqVmeAddr = VmeWindowParams->VmeBusAddr - VME_A32UD_BASE;
            break;
        case V_A32SD:
            ReqVmeAddr = VmeWindowParams->VmeBusAddr - VME_A32SD_BASE;
            break;

        default: /* valid access mode already tested */
            break;
    } /* end switch */

    VmeWindowParams->PhysAddr = ( ImageBaseAddr + ReqVmeAddr ) & LOCAL_PG_MSK;

    if ( VmeWindowParams->SysModeFlag == UVI_KERNEL )
    {
        PtrToRequestedWindow = (unsigned char *)ioremap_nocache
                             ( ImageBaseAddr + ReqVmeAddr, VmeWindowParams->ReqWndSize );

        if ( PtrToRequestedWindow != NULL )
        {
            VmeWindowParams->PtrToVmeWindow = PtrToRequestedWindow;
        }
        else
        {
            WHENDEBUG( UVI_DBERROR )
            {
                printk( KERN_ERR "%s:%s; Did not get a pointer to the device window.\n",
                        MODULE_NAME, ThisFunction );
            }
            FuncStatus = UVI_NULL_PTR;
            goto ReturnStatus;
        }
    }

ReturnStatus:
/* Make sure we release the lock before leaving. */
/*    spin_unlock( &WndOp_SpinLock ); */

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    return( FuncStatus );

} /* end of 'uviGetVmeWindow' */


/********************************************************************************
*
*  FUNCTION:   uviReleaseVmeWindow
*
*  PURPOSE:    To free up the VME window that is no longer needed.
*
*  PARAMETERS: Pointer to VME window parameters structure, VmeWindowParams
*
*  RETURNS:    UVI_SUCCESS   : Success
*              UVI_UNMAP_ERR : Problem unmapping 'window', may have null pointer
*
*  GLOBALS:
*
********************************************************************************/
UVI_INT32 uviReleaseVmeWindow( struct uviVmeWindowParameters *VmeWindowParams )
{

    static char *ThisFunction = "uviReleaseVmeWindow";

    UVI_BOOL DeviceReleased;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

    if ( VmeWindowParams->SysModeFlag == UVI_USER )
    {
        WHENDEBUG( UVI_DBEXTRA )
        {
            printk( KERN_INFO "%s:%s; Release request comes from user space, do nothing.\n",
                    MODULE_NAME, ThisFunction );
        }
        return( UVI_SUCCESS );
    }

    WHENDEBUG( UVI_DBEXTRA )
    {
        printk( KERN_INFO "%s:%s; Device address is %x.\n", MODULE_NAME, ThisFunction,
                VmeWindowParams->VmeBusAddr );
        printk( KERN_INFO "%s:%s; Addr Mod is 0x%x.\n", MODULE_NAME, ThisFunction,
                VmeWindowParams->AddrMod );
    }

/*
** For a kernel level release request, iounmap the area given when the window was
** obtained.  The user must give us back the original pointer that was returned.
*/
    WHENDEBUG( UVI_DBEXTRA )
    {
        printk( KERN_INFO "%s:%s; Pointer to area to unmap is %x.\n", MODULE_NAME,
                ThisFunction, (UVI_UINT32)VmeWindowParams->PtrToVmeWindow );
    }

    if ( VmeWindowParams->PtrToVmeWindow != NULL )
    {
        iounmap( (unsigned char *)( VmeWindowParams->PtrToVmeWindow ) );
        DeviceReleased = UVI_TRUE;
    }
    else
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; A problem occured trying to unmap window area.\n",
                    MODULE_NAME, ThisFunction );
        }
        DeviceReleased = UVI_FALSE;
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    if ( DeviceReleased )
    {
        return( UVI_SUCCESS );
    }
    else
    {
        return( UVI_UNMAP_ERR );
    }

} /* end of 'uviReleaseVmeWindow' */


/********************************************************************************
*
*  FUNCTION:   uviProbeAddress
*
*  PURPOSE:    To check if an address access returns an error.  Presently supported
*              for kernel level ( module to module ) requests only.
*
*  PARAMETERS: Pointer to VME window parameters structure, VmeWindowParams; address
*              access width, AddrWidth
*
*  RETURNS:    UVI_SUCCESS       : Success
*              UVI_INV_WIDTH     : Invalid access width
*              UVI_WNDSPEC_ERROR : VME 'window' specification error
*              UVI_INV_ADDRMOD   : Invalid/unsupported address modifier
*              MODUVI_NULL_PTR   : pointer not obtained
*              UVI_UNMAP_ERR     : Problem unmapping 'window', may have null pointer
*
*  GLOBALS:    IsaCmdRegVal, FuncStatus, PlxFound, PlxMappedAddr, VmeBerrStatus,
*              
*
********************************************************************************/
UVI_INT32 uviProbeAddress( struct uviVmeWindowParameters *VmeWindowParams,
                           UVI_INT16 AddrWidth )
{

    static char *ThisFunction = "uviProbeAddress";

    volatile unsigned char *SavedWindowPtr;

    UVI_UINT32 GetPtrStatus,
               RelPtrStatus,
               AddrAccessChk,
               SavedWindowSize;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

    FuncStatus = UVI_NOBERR;
/*
** Clear any BERRs that may have occured for other reasons from other operations,
** such as DMA ops.  If a previous operation generated a BERR, then it must have
** already been handled in some way.
*/
    if ( PlxFound )
    {
        *(UVI_UINT32 *)( PlxMappedAddr + PLX_B_INT_STATUS ) |= BERR_S_PLX;
    }
    else
/* use the COMM reg. */
    {
        IsaCmdRegVal |= BERRST_CLR_NOTPLX;
        writel( IsaCmdRegVal, NOTPLX_CMDREG_ADDR );
    }

/* Save users pointer to window before performing an access check. */
    SavedWindowPtr = VmeWindowParams->PtrToVmeWindow;

/*
** Save users window size.  We do this because we need to make sure that the size
** used to get a pointer to a VME window is never less than a longword.
*/
    SavedWindowSize = VmeWindowParams->ReqWndSize;
    VmeWindowParams->ReqWndSize = PAGE_SIZE;

/* Use existing function to get a pointer for an access check. */
    GetPtrStatus = uviGetVmeWindow( VmeWindowParams );
    if ( GetPtrStatus != UVI_SUCCESS )
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; Call to get device window failed.\n",
                    MODULE_NAME, ThisFunction );
        }
        FuncStatus = GetPtrStatus;
        goto ReturnStatus;
    }

/* Make the address check. */
    switch( AddrWidth )
    {
        case UVI_BYTE:
            AddrAccessChk = *( VmeWindowParams->PtrToVmeWindow );
            break;
        case UVI_WORD:
            AddrAccessChk = *( (UVI_UINT16 *)( VmeWindowParams->PtrToVmeWindow ) );
            break;
        case UVI_LWORD:
            AddrAccessChk = *( (UVI_UINT32 *)( VmeWindowParams->PtrToVmeWindow ) );
            break;

        default:
            WHENDEBUG( UVI_DBERROR )
            {
                printk( KERN_ERR "%s:%s; Address width invalid. 0x%x.\n", MODULE_NAME,
                        ThisFunction, AddrWidth );
            }
            FuncStatus = UVI_INV_WIDTH;
            goto ReturnStatus;
            break;
    } /* end 'switch' on address width */

/* Release the 'window' used for the access check. */
    RelPtrStatus = uviReleaseVmeWindow( VmeWindowParams );
    if ( RelPtrStatus != UVI_SUCCESS )
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; Call to release device window failed.\n",
                    MODULE_NAME, ThisFunction );
        }
        FuncStatus = RelPtrStatus;
        goto ReturnStatus;
    }

/* Check for a bus error on address access ( for a 'kernel' module ). */
    if ( ( VmeWindowParams->SysModeFlag == UVI_KERNEL ) &&
                                            ( VmeWindowParams->PtrToVmeWindow != NULL ) )
    {
        if ( PlxFound )
        {
            VmeBerrStatus = *(UVI_UINT32 *)( PlxMappedAddr + PLX_B_INT_STATUS );

            if ( VmeBerrStatus & BERR_S_PLX )
            {
                WHENDEBUG( UVI_DBEXTRA )
                {
                    printk( KERN_INFO "%s:%s; Probe generated a BERR.\n",
                            MODULE_NAME, ThisFunction );
                }
                FuncStatus = UVI_BERR;
                VmeBerrStatus |= BERR_S_PLX; /* clear BERR */
                *(UVI_UINT32 *)( PlxMappedAddr + PLX_B_INT_STATUS ) = VmeBerrStatus;
            }
        }
        else
/* use the COMM reg. */
        {
            WHENDEBUG( UVI_DBEXTRA )
            {
                printk( KERN_INFO "%s:%s; we are using the comm register.\n",
                        MODULE_NAME, ThisFunction );
            }
            IsaCmdRegVal = readw( NOTPLX_CMDREG_ADDR );
            VmeBerrStatus = IsaCmdRegVal & BERRST_CLR_NOTPLX;

            if ( VmeBerrStatus )
            {
                WHENDEBUG( UVI_DBEXTRA )
                {
                    printk( KERN_INFO "%s:%s; Probe generated a BERR.\n",
                            MODULE_NAME, ThisFunction );
                }
                FuncStatus = UVI_BERR;
                IsaCmdRegVal |= BERRST_CLR_NOTPLX; /* clear BERR */
                writew( IsaCmdRegVal, NOTPLX_CMDREG_ADDR );
            }
        }
    }

ReturnStatus:
/* Restore the users window pointer and requested window size. */
    VmeWindowParams->PtrToVmeWindow = SavedWindowPtr;
    VmeWindowParams->ReqWndSize = SavedWindowSize;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    return( FuncStatus );

} /* end of 'uviProbeAddress' */


/********************************************************************************
*
*  FUNCTION:   uviGetUniverseInterruptLevel
*
*  PURPOSE:    To find out what interrupt level the universe device uses.
*
*  PARAMETERS: Pointer to interrupt level data
*
*  RETURNS:    UVI_SUCCESS : Success
*              No meaningful error return yet
*
*  GLOBALS:    UnivIrq
*
********************************************************************************/
UVI_INT32 uviGetUniverseInterruptLevel( UVI_UINT32 *UnivIntrLvl )
{

    static char *ThisFunction = "uviGetUniverseInterruptLevel";
  
    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

    *UnivIntrLvl = UnivIrq;
    WHENDEBUG( UVI_DBINTR )
    {
        printk( KERN_INFO "%s:%s; universe interrupt level is 0x%x.\n", MODULE_NAME,
                ThisFunction, *UnivIntrLvl );
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    return( UVI_SUCCESS );

} /* end of 'uviGetUniverseInterruptLevel' */


/********************************************************************************
*
*  FUNCTION:   uviInstallUniverseInterruptHandler
*
*  PURPOSE:    To install the local universe interrupt handler.
*
*  PARAMETERS: None
*
*  RETURNS:    UVI_SUCCESS        : Success
*              UVI_REGINTR_FAILED : Failed to register the handler
*
*  GLOBALS:    UnivIntrInUse, DevId, UnivIrq
*
********************************************************************************/
UVI_INT32 uviInstallUniverseInterruptHandler( void )
{

    static char *ThisFunction = "uviInsUnivIntrHdlr";

    UVI_INT32 ReqIntrStatus;
 
    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

/*
** Register our local universe interrupt handler, if it hasn't already been
** registered ( installed ).
*/
    if ( UnivIntrInUse == 0 )
    {
        UnivInterruptId = UVI_INTRID;
        DevId = &UnivInterruptId;

        ReqIntrStatus = request_irq( UnivIrq,
                                     uviUniverseIntrHandlr,
                                     SA_INTERRUPT | SA_SHIRQ,
                                     "uviIntrHandlr",
                                     DevId );

        if ( ReqIntrStatus != UVI_SUCCESS )
        {
            WHENDEBUG( UVI_DBERROR )
            {
                printk( KERN_ERR "%s:%s; Failed to register local interrupt handler,"
                        " this is a problem!\n", MODULE_NAME, ThisFunction );
            }
            return( UVI_REGINTR_FAILED );
        }

    }
 
    ++UnivIntrInUse;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    return( UVI_SUCCESS );

} /* end of 'uviInstallUniverseInterruptHandler' */


/********************************************************************************
*
*  FUNCTION:   uviRemoveUniverseInterruptHandler
*
*  PURPOSE:    To remove the local universe interrupt handler.
*
*  PARAMETERS: None
*
*  RETURNS:    UVI_SUCCESS      : Success
*              Don't have an error status yet
*
*  GLOBALS:    UnivIntrInUse, UnivIrq, DevId
*
********************************************************************************/
UVI_INT32 uviRemoveUniverseInterruptHandler( void )
{

    static char *ThisFunction = "uviRemUnivIntrHdlr";
 
    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

/*
** Free our local interrupt handler if the 'use' count is 0 indicating no other
** module ( including ourself ) is using the universe handler.
*/

    if ( UnivIntrInUse > 1 )
    {
        --UnivIntrInUse;
    }
    else
    {
        /* clear any previous DMA status  and disable interrupt in LINT_EN reg */
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = 0x100;
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= 0x00FFFEFF;

        free_irq( UnivIrq, DevId );
        --UnivIntrInUse;
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    return( UVI_SUCCESS );

} /* end of 'uviRemoveUniverseInterruptHandler' */


/********************************************************************************
*
*  FUNCTION:   uviEnableDmaInterrupt
*
*  PURPOSE:    To enable the dma interrupt
*
*  PARAMETERS: NONE 
*  RETURNS:    UVI_SUCCESS      : Success
*
*  GLOBALS:    UviBaseAddr
*
********************************************************************************/
UVI_INT32 uviEnableDmaInterrupt( void )
{

    static char *ThisFunction = "uviEnableDmaInterrupt";
  
    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

/* clear any previous DMA status, set MAP1 for INTA and enable interrupt in LINT_EN reg */
    *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = 0x700; /* R/W1 to clear */
    *(UVI_UINT32 *)( UnivBaseAddr + LINT_MAP1 ) &= 0x77770770; /* 0 = INTA, 1 = NMI */
    *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= 0x100; /* enable DMA interrupt */

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME, 
                                     ThisFunction );

    return( UVI_SUCCESS );

} /* end uviEnableDmaInterrupt */


/********************************************************************************
*
*  FUNCTION:   uviDisableDmaInterrupt
*
*  PURPOSE:    To disable the dma interrupt
*
*  PARAMETERS: NONE 
*  RETURNS:    UVI_SUCCESS      : Success
*
*  GLOBALS:    UviBaseAddr
*
********************************************************************************/
UVI_INT32 uviDisableDmaInterrupt( void )
{

    static char *ThisFunction = "uviDisableDmaInterrupt";
  
    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

    /* clear any previous DMA status and disable interrupt in LINT_EN reg */
    *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= ~(0x100); /* disble DMA interrupt */
    *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = 0x100; /* R/W1 to clear */

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME, 
                                     ThisFunction );

    return( UVI_SUCCESS );

} /* end uviDisableDmaInterrupt */


/********************************************************************************
*
*  FUNCTION:   uviEnableInterrupt
*
*  PURPOSE:    To enable a vme interrupt level for a module/user request, and to
*              keep track of the number of 'users' of any given level.
*
*  PARAMETERS: Requested interrupt level, VmeIntLvl; requested vector, VmeIntVector
* 
*  RETURNS:    UVI_SUCCESS      : Success
*              UVI_INV_LVL_VECT : Invalid level or vector passed in
*
*  GLOBALS:    IntTable, LvlIndex, UnivBaseAddr, LvlInUseCnt
*
********************************************************************************/
UVI_INT32 uviEnableInterrupt( UVI_INT32 VmeIntLvl, UVI_INT32 VmeIntVector )
{

    static char *ThisFunction = "uviEnableInterrupt";
  
    UVI_INT8 LvlIndex;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

/* Lock function while in use.  Responsibility moved to handler. */
/*    spin_lock_irqsave( &IrqOp_SpinLock, Flags ); */

/* Make sure we have a valid interrupt level and vector to work with. */
    if ( ( VmeIntLvl < 1 || VmeIntLvl > MAX_VINT_LVLS ) ||
           ( VmeIntVector < 0 || VmeIntVector > MAX_VINT_VECTORS ) ||
             ( VmeIntVector != ( VmeIntVector & 0xFE ) ) )
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; Invalid level or vector, 0x%x, 0x%x.\n", MODULE_NAME,
                    ThisFunction, VmeIntLvl, VmeIntVector );
        }
        return( UVI_INV_LVL_VECT );
    }

/* The Universe type 1 device has a bug that does not allow it to handle interrupts
** on multiple levels.  Add code to allow only a single level of interrupt if a type
** 1 device is in use.
*/

/*
** Check to see if the requested level is already enabled.  If it is, make sure that
** the requested vector is not already in use.  If it is, return an error, otherwise,
** set the flag for this vector and increment a 'use' count for this level.  If the
** level is not enabled yet, then set the vector flag, enable the level, and increment
** the 'use' count.
*/
    LvlIndex = VmeIntLvl - 1;
    if ( ( *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) ) & LINT_EN_VIRQ( VmeIntLvl ) )
    {
        if ( IntTable[LvlIndex][VmeIntVector].IntrFlags & VECTOR_IN_USE ) /* vector in use */
        {
            return( UVI_INTR_IN_USE );
        }
        else
        {
            IntTable[LvlIndex][VmeIntVector].IntrFlags |= VECTOR_IN_USE;
            LvlInUseCnt[LvlIndex] += 1;
        }
    }
    else
    {
        IntTable[LvlIndex][VmeIntVector].IntrFlags |= VECTOR_IN_USE;

        *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= LINT_EN_VIRQ( VmeIntLvl );
        LvlInUseCnt[LvlIndex] += 1;
    }

/* Make sure we release the lock before leaving.  Responsibility moved to handler. */
/*    spin_unlock_irqrestore( &IrqOp_SpinLock, Flags ); */
    WHENDEBUG( UVI_DBINTR )
    {
        printk( KERN_INFO "%s:%s; level/vector/flag info is %x/%x/%x.\n",
                MODULE_NAME, ThisFunction, LvlIndex + 1, VmeIntVector,
                IntTable[LvlIndex][VmeIntVector].IntrFlags );
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    return( UVI_SUCCESS );

} /* end of 'uviEnableInterrupt' */


/********************************************************************************
*
*  FUNCTION:   uviCheckIntSource
*
*  PURPOSE:    To determine if the interrupt is for the caller.
*
*  PARAMETERS: VmeIntLvl, VmeIntVector
*
*  RETURNS:    UVI_INV_LVL_VECT   : Invalid level or vector
*              UVI_INTR_YOURS     : Interrupt is callers 
*              UVI_INTR_NOT_YOURS : Interrupt not callers
*
*  GLOBALS:    UnivBaseAddr
*
********************************************************************************/
UVI_INT32 uviCheckIntSource( UVI_INT32 VmeIntLvl, UVI_INT32 VmeIntVector )
{

    static char *ThisFunction = "uviCheckIntSource";

    UVI_INT8 LvlIndex;

    UVI_UINT32 ItsForYou = UVI_FALSE,
               IntLvl,
               VectorId;

    WHENDEBUG( UVI_DBINTR ) printk( KERN_CRIT "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

/* Lock function while in use.  Responsibility moved to handler. */
/*    spin_lock_irqsave( &IrqOp_SpinLock, Flags ); */

/* Make sure we have a valid interrupt level and vector to work with. */
    if ( ( VmeIntLvl < 1 || VmeIntLvl > MAX_VINT_LVLS ) ||
           ( VmeIntVector < 0 || VmeIntVector > MAX_VINT_VECTORS ) ||
             ( VmeIntVector != ( VmeIntVector & 0xFE ) ) )
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; Invalid level or vector, 0x%x, 0x%x.\n", MODULE_NAME,
                    ThisFunction, VmeIntLvl, VmeIntVector );
        }
        return( UVI_INV_LVL_VECT );
    }

/*
** Read the status ID for the requested level and compare to the vector passed in.
** If we have a match, then the interrupt is for the caller.
*/
    LvlIndex = VmeIntLvl - 1;
    VectorId = *(UVI_UINT32 *)( UnivBaseAddr + ( V_STATID_BASE +
                                                 ( V_STATID_OFFS * LvlIndex ) ) );
    IntLvl = (UVI_UINT32)( 1 << VmeIntLvl );
    if ( ( *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) & IntLvl ) &&
                                                           ( VectorId == VmeIntVector ) )
    {
        ItsForYou = UVI_TRUE;
    }
    else
    {
        ItsForYou = UVI_FALSE;
    }

/* Make sure we release the lock before leaving.  Responsibility moved to handler. */
/*    spin_unlock_irqrestore( &IrqOp_SpinLock, Flags ); */

    WHENDEBUG( UVI_DBINTR ) printk( KERN_CRIT "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );
    if ( ItsForYou )
    {
        return( UVI_INTR_YOURS );
    }
    else
    {
        return( UVI_INTR_NOT_YOURS );
    } 

} /* end of 'uviCheckIntSource' */


/********************************************************************************
*
*  FUNCTION:   uviReArmInterrupt
*
*  PURPOSE:    To re-arm an interrupt that a caller has finished handling.
*
*  PARAMETERS: VmeIntLvl, VmeIntVector
* 
*  RETURNS:    UVI_SUCCESS      : Success
*              UVI_INV_LVL_VECT : Invalid level or vector
*
*  GLOBALS:    UnivBaseAddr, LvlIndex
*
********************************************************************************/
UVI_INT32 uviReArmInterrupt( UVI_INT32 VmeIntLvl, UVI_INT32 VmeIntVector )
{

    static char *ThisFunction = "uviReArmInterrupt";

    UVI_INT8 LvlIndex;

    WHENDEBUG( UVI_DBINTR ) printk( KERN_CRIT "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

/* Lock function while in use.  Responsibility moved to handler. */
/*    spin_lock_irqsave( &IrqOp_SpinLock, Flags ); */

/* Make sure we have a valid interrupt level and vector to work with. */
    if ( ( VmeIntLvl < 1 || VmeIntLvl > MAX_VINT_LVLS ) ||
           ( VmeIntVector < 0 || VmeIntVector > MAX_VINT_VECTORS ) ||
             ( VmeIntVector != ( VmeIntVector & 0xFE ) ) )
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; Invalid level or vector, 0x%x, 0x%x.\n", MODULE_NAME,
                    ThisFunction, VmeIntLvl, VmeIntVector );
        }
        return( UVI_INV_LVL_VECT );
    }

/* Re-arm this interrupt as the caller is done with it. */
    LvlIndex = VmeIntLvl - 1;

/*
** Clear the interrupt on vme request line.  Write a one to the register bit that
** corresponds to the active level.
*/
    *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = ( 1 << VmeIntLvl );
/*    *(UVI_UINT32 *)( UnivBaseAddr + VINT_STAT ) = ( 1 << LvlIndex ); */

/* Make sure we release the lock before leaving.  Responsibility moved to handler. */
/*    spin_unlock_irqrestore( &IrqOp_SpinLock, Flags ); */

    WHENDEBUG( UVI_DBINTR ) printk( KERN_CRIT "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    return( UVI_SUCCESS );

} /* end of 'uviReArmInterrupt' */


/********************************************************************************
*
*  FUNCTION:   uviDisableInterrupt
*
*  PURPOSE:    To disable a vme interrupt level at a module/user request if the
*              level is no longer in use.
*
*  PARAMETERS: VmeIntLvl, VmeIntVector
*
*  RETURNS:    UVI_SUCCESS      : Success
*              UVI_INV_LVL_VECT : Invalid level or vector
*
*  GLOBALS:    LvlIndex, LvlInUseCnt, UnivBaseAddr, IntTable, Univ1Found
*
********************************************************************************/
UVI_INT32 uviDisableInterrupt( UVI_INT32 VmeIntLvl, UVI_INT32 VmeIntVector )
{

    static char *ThisFunction = "uviDisableInterrupt";

    UVI_INT8 LvlIndex;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

/* Lock function while in use.  Responsibility moved to handler. */
/*    spin_lock_irqsave( &IrqOp_SpinLock, Flags ); */

/* Make sure we have a valid interrupt level and vector to work with. */
    if ( ( VmeIntLvl < 1 || VmeIntLvl > MAX_VINT_LVLS ) ||
           ( VmeIntVector < 0 || VmeIntVector > MAX_VINT_VECTORS ) ||
             ( VmeIntVector != ( VmeIntVector & 0xFE ) ) )
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; Invalid level or vector, 0x%x, 0x%x.\n", MODULE_NAME,
                    ThisFunction, VmeIntLvl, VmeIntVector );
        }
        return( UVI_INV_LVL_VECT );
    }

/*
** If the interrupt level is still 'in use' then we can not disable it.  We will
** clear the vector for this user and return a meaningful status.
*/
    LvlIndex = VmeIntLvl - 1;
    LvlInUseCnt[LvlIndex] -= 1;
    if ( LvlInUseCnt[LvlIndex] != 0 ) 
    {
        IntTable[LvlIndex][VmeIntVector].IntrFlags &= ~VECTOR_IN_USE;
    }
    else
    {
        IntTable[LvlIndex][VmeIntVector].IntrFlags &= ~VECTOR_IN_USE;
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= LINT_DIS_VIRQ( VmeIntLvl );

        if ( Univ1Found )
        {
            *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) |= ( 1 << VmeIntLvl );
/*            *(UVI_UINT32 *)( UnivBaseAddr + VINT_STAT ) |= ( 1 << VmeIntLvl ); */
        }
        else
        {
            *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) |= ( 1 << VmeIntLvl );
/*            *(UVI_UINT32 *)( UnivBaseAddr + VINT_STAT ) |= ( 1 << VmeIntLvl ); */
        }

        WHENDEBUG( UVI_DBINTR )
        {
            printk( KERN_INFO "%s:%s; level/vector/flag info is %x/%x/%x.\n", MODULE_NAME,
                    ThisFunction, LvlIndex + 1, VmeIntVector,
                    IntTable[LvlIndex][VmeIntVector].IntrFlags );
        }
     }

/* Make sure we release the lock before leaving.  Responsibility moved to handler. */
/*    spin_unlock_irqrestore( &IrqOp_SpinLock, Flags ); */

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    return( UVI_SUCCESS );

} /* end of 'uviDisableInterrupt' */


/********************************************************************************
*
*  FUNCTION:   uviUniverseIntrHandlr
*
*  PURPOSE:    Handle interrupts that the universe module itself is responsible
*              for.
*
*  PARAMETERS: irq, *dev_id, *regs; OS passes these in
*
*  RETURNS:    None
*
*  GLOBALS:    UnivBaseAddr, Flags, dma_wq
*
********************************************************************************/
void uviUniverseIntrHandlr( int irq, void *dev_id, struct pt_regs *regs )
{

    static char *ThisFunction = "uviUnivIntrHandlr";

    UVI_UINT32 Lint_en, Lint_stat, tmp, Vme_vector, index;

    WHENDEBUG( UVI_DBINTR ) printk( KERN_CRIT "%s:%s; Entering ...\n", MODULE_NAME,
                                    ThisFunction );

    /* Lock function while in use. */
    spin_lock_irqsave( &IrqOp_SpinLock, Flags );

    /* save LINT_STAT / LINT_EN info */
    Lint_en = *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN );
    Lint_stat = *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT );

    /* check LINT_STAT / LINT_EN for VME INT */
    tmp = Lint_en & LINT_VME_INT;
    tmp &= Lint_stat;

    WHENDEBUG( UVI_DBINTR ) printk( KERN_CRIT "%s:%s; Lint_en:%X Lint_stat:%X tmp:%X\n",
                                   MODULE_NAME, ThisFunction, Lint_en, Lint_stat, tmp );
    if( tmp )
    {
      if( tmp & LINT_EN_VIRQ( 7 ) )
      {
        /* write a 1 to clear the status R/W1C and get the vector */
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_EN_VIRQ( 7 ); 
        Vme_vector = *(UVI_UINT32 *)( UnivBaseAddr + V7_STATID );

        if( Vme_vector & IACK_BUS_ERROR )
        {
           /* wake up all tasks for this level since the vector was invalid */
           for( index = 0; index < 0x100; index++ )
           {
             if( (IntTable[6][index].IntrFlags & (USER_OWNS_INT | USER_READY_INT)) ==
                 (USER_OWNS_INT | USER_READY_INT) )
             {
               wake_up_interruptible( &int_wq[6][index] );
             }
           }
        }
        else
        {
           /* wake up only the one that owns the vector */
           if( (IntTable[6][Vme_vector & 0xFF].IntrFlags & (USER_OWNS_INT | USER_READY_INT)) ==
               (USER_OWNS_INT | USER_READY_INT) )
           {
             wake_up_interruptible( &int_wq[6][Vme_vector & 0xFF] );
           }
        }
      }

      if( tmp & LINT_EN_VIRQ( 6 ) )
      {
        /* write a 1 to clear the status R/W1C and get the vector */
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_EN_VIRQ( 6 ); 
        Vme_vector = *(UVI_UINT32 *)( UnivBaseAddr + V6_STATID );

        if( Vme_vector & IACK_BUS_ERROR )
        {
           /* wake up all tasks for this level since the vector was invalid */
           for( index = 0; index < 0x100; index++ )
           {
             if( (IntTable[5][index].IntrFlags & (USER_OWNS_INT | USER_READY_INT)) ==
                 (USER_OWNS_INT | USER_READY_INT) )
             {
               wake_up_interruptible( &int_wq[5][index] );
             }
           }
        }
        else
        {
           /* wake up only the one that owns the vector */
           if( (IntTable[5][Vme_vector & 0xFF].IntrFlags & (USER_OWNS_INT | USER_READY_INT)) ==
               (USER_OWNS_INT | USER_READY_INT) )
           {
             wake_up_interruptible( &int_wq[5][Vme_vector & 0xFF] );
           }
        }
      }

      if( tmp & LINT_EN_VIRQ( 5 ) )
      {
        /* write a 1 to clear the status R/W1C and get the vector */
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_EN_VIRQ( 5 ); 
        Vme_vector = *(UVI_UINT32 *)( UnivBaseAddr + V5_STATID );

        if( Vme_vector & IACK_BUS_ERROR )
        {
           /* wake up all tasks for this level since the vector was invalid */
           for( index = 0; index < 0x100; index++ )
           {
             if( (IntTable[4][index].IntrFlags & (USER_OWNS_INT | USER_READY_INT)) ==
                 (USER_OWNS_INT | USER_READY_INT) )
             {
               wake_up_interruptible( &int_wq[4][index] );
             }
           }
        }
        else
        {
           /* wake up only the one that owns the vector */
           if( (IntTable[4][Vme_vector & 0xFF].IntrFlags & (USER_OWNS_INT | USER_READY_INT)) ==
               (USER_OWNS_INT | USER_READY_INT) )
           {
             wake_up_interruptible( &int_wq[4][Vme_vector & 0xFF] );
           }
        }
      }

      if( tmp & LINT_EN_VIRQ( 4 ) )
      {
        /* write a 1 to clear the status R/W1C and get the vector */
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_EN_VIRQ( 4 ); 
        Vme_vector = *(UVI_UINT32 *)( UnivBaseAddr + V4_STATID );

        if( Vme_vector & IACK_BUS_ERROR )
        {
           /* wake up all tasks for this level since the vector was invalid */
           for( index = 0; index < 0x100; index++ )
           {
             if( (IntTable[3][index].IntrFlags & (USER_OWNS_INT | USER_READY_INT)) ==
                 (USER_OWNS_INT | USER_READY_INT) )
             {
               wake_up_interruptible( &int_wq[3][index] );
             }
           }
        }
        else
        {
           /* wake up only the one that owns the vector */
           if( (IntTable[3][Vme_vector & 0xFF].IntrFlags & (USER_OWNS_INT | USER_READY_INT)) ==
               (USER_OWNS_INT | USER_READY_INT) )
           {
             wake_up_interruptible( &int_wq[3][Vme_vector & 0xFF] );
           }
        }
      }

      if( tmp & LINT_EN_VIRQ( 3 ) )
      {
        /* write a 1 to clear the status R/W1C and get the vector */
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_EN_VIRQ( 3 ); 
        Vme_vector = *(UVI_UINT32 *)( UnivBaseAddr + V3_STATID );

        if( Vme_vector & IACK_BUS_ERROR )
        {
           /* wake up all tasks for this level since the vector was invalid */
           for( index = 0; index < 0x100; index++ )
           {
             if( (IntTable[2][index].IntrFlags & (USER_OWNS_INT | USER_READY_INT)) ==
                 (USER_OWNS_INT | USER_READY_INT) )
             {
               wake_up_interruptible( &int_wq[2][index] );
             }
           }
        }
        else
        {
           /* wake up only the one that owns the vector */
           if( (IntTable[2][Vme_vector & 0xFF].IntrFlags & (USER_OWNS_INT | USER_READY_INT)) ==
               (USER_OWNS_INT | USER_READY_INT) )
           {
             wake_up_interruptible( &int_wq[2][Vme_vector & 0xFF] );
           }
        }
      }

      if( tmp & LINT_EN_VIRQ( 2 ) )
      {
        /* write a 1 to clear the status R/W1C and get the vector */
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_EN_VIRQ( 2 ); 
        Vme_vector = *(UVI_UINT32 *)( UnivBaseAddr + V2_STATID );

        if( Vme_vector & IACK_BUS_ERROR )
        {
           /* wake up all tasks for this level since the vector was invalid */
           for( index = 0; index < 0x100; index++ )
           {
             if( (IntTable[1][index].IntrFlags & (USER_OWNS_INT | USER_READY_INT)) ==
                 (USER_OWNS_INT | USER_READY_INT) )
             {
               wake_up_interruptible( &int_wq[1][index] );
             }
           }
        }
        else
        {
           /* wake up only the one that owns the vector */
           if( (IntTable[1][Vme_vector & 0xFF].IntrFlags & (USER_OWNS_INT | USER_READY_INT)) ==
               (USER_OWNS_INT | USER_READY_INT) )
           {
             wake_up_interruptible( &int_wq[1][Vme_vector & 0xFF] );
           }
        }
      }

      if( tmp & LINT_EN_VIRQ( 1 ) )
      {
        /* write a 1 to clear the status R/W1C and get the vector */
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_EN_VIRQ( 1 ); 
        Vme_vector = *(UVI_UINT32 *)( UnivBaseAddr + V1_STATID );

        if( Vme_vector & IACK_BUS_ERROR )
        {
           /* wake up all tasks for this level since the vector was invalid */
           for( index = 0; index < 0x100; index++ )
           {
             if( (IntTable[0][index].IntrFlags & (USER_OWNS_INT | USER_READY_INT)) ==
                 (USER_OWNS_INT | USER_READY_INT) )
             {
               wake_up_interruptible( &int_wq[0][index] );
             }
           }
        }
        else
        {
           /* wake up only the one that owns the vector */
           if( (IntTable[0][Vme_vector & 0xFF].IntrFlags & (USER_OWNS_INT | USER_READY_INT)) ==
               (USER_OWNS_INT | USER_READY_INT) )
           {
             wake_up_interruptible( &int_wq[0][Vme_vector & 0xFF] );
           }
        }
      }
    } /* end if VME INT/STAT */

    /* check LINT_STAT / LINT_EN for DMA and user own */
    if( (Lint_en & LINT_DMA) && (Lint_stat & LINT_DMA)  && (UserOwnDma & USER_OWNS_DMA) )
    {
      /* Clear the local DMA interrupt. */
      if ( *(UVI_UINT32 *)( UnivBaseAddr + DGCS ) & 0x6F00 )
      {
          /* disable DMA int., clear DMA status, interrupt sleeping process */ 
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= 0xFFF6FF; 
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_DMA; 
          wake_up_interruptible( &dma_wq ); 
      }
    }

    /* check LINT_STAT / LINT_EN for MB and user own */
    if( (Lint_en & (LINT_MB0 | LINT_MB1 | LINT_MB2 | LINT_MB3))
         && (Lint_stat & (LINT_MB0 | LINT_MB1 | LINT_MB2 | LINT_MB3))
         && (UserOwnMB0 | UserOwnMB1 | UserOwnMB2 | UserOwnMB3) )
    {  
        WHENDEBUG( UVI_DBINTR ) printk( KERN_CRIT "%s:%s; OWN: %X Entering ...\n",
          MODULE_NAME, ThisFunction, ((UserOwnMB0 | UserOwnMB1 | UserOwnMB2 | UserOwnMB3) ));

        if( UserOwnMB0 && (Lint_en & LINT_MB0) && (Lint_stat & LINT_MB0) )   
        {  
          /* clear MB0 status, interrupt sleeping process */
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= ~LINT_MB0;
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_MB0;
          wake_up_interruptible( &mb0_wq );
        }

        if( UserOwnMB1 && (Lint_en & LINT_MB1) && (Lint_stat & LINT_MB1) )
        {  
          /* clear MB1 status, interrupt sleeping process */
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= ~LINT_MB1;
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_MB1;
          wake_up_interruptible( &mb1_wq );
        }

        if( UserOwnMB2 && (Lint_en & LINT_MB2) && (Lint_stat & LINT_MB2) )
        {  
          /* clear MB2 status, interrupt sleeping process */
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= ~LINT_MB2;
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_MB2;
          wake_up_interruptible( &mb2_wq );
        }

        if( UserOwnMB3 && (Lint_en & LINT_MB3) && (Lint_stat & LINT_MB3) )
        {  
          /* clear MB3 status, interrupt sleeping process */
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= ~LINT_MB3;
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_MB3;
          wake_up_interruptible( &mb3_wq );
        }

    } /* end if MB interrupt */


    /* Make sure we release the lock before leaving. */
    spin_unlock_irqrestore( &IrqOp_SpinLock, Flags );

    WHENDEBUG( UVI_DBINTR ) printk( KERN_CRIT "%s:%s; Exiting ...\n", MODULE_NAME,
                                    ThisFunction );

    return;

} /* end 'uviUniverseIntrHandlr' */


/********************************************************************************
*
*  FUNCTION:   uviRequestUseVmeInt
*
*  PURPOSE:    To lock the VME interrupt level/vector resource if available.
*              This does NOT enable interrupts.
*
*  PARAMETERS: vmelevel - vme interrupt level to use
*              vmevector - vme interrupt vector to use
*
*  RETURNS:    UVI_SUCCESS      : Success
*              UVI_INT_BUSY
*              UVI_BAD_PARAMETER_1
*              UVI_BAD_PARAMETER_2
*
*  GLOBALS:    IntTable, LvlInUseCnt
*
********************************************************************************/
UVI_INT32 uviRequestUseVmeInt( UVI_INT32 vmelevel, UVI_INT32 vmevector )
{

    UVI_INT32 retvalue = UVI_SUCCESS;

    static char *ThisFunction = "uviRequestUseVmeInt";

    /* Lock function while in use. */
    spin_lock_irqsave( &IrqOp_SpinLock, Flags );

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; level:%x vector: %x Entering ...\n",
                                     MODULE_NAME, ThisFunction, vmelevel, vmevector );

    /* check for valid level and vector */
    if( (vmelevel < 1) || (vmelevel > 7) )
    {
      retvalue = UVI_BAD_PARAMETER_1;
    }
    else if( (vmevector < 0) || (vmevector > 0xFF) )
    {
      retvalue = UVI_BAD_PARAMETER_2;
    }
    else if ( IntTable[vmelevel - 1][vmevector].IntrFlags & VECTOR_IN_USE ) /* vector in use */
    {
        retvalue = UVI_INTR_IN_USE;
    }
    else
    {
        IntTable[vmelevel - 1][vmevector].IntrFlags |= VECTOR_IN_USE;
        LvlInUseCnt[vmelevel - 1] += 1;
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; STAT: %i Exiting ...\n",
                                     MODULE_NAME, ThisFunction, retvalue );

    /* Make sure we release the lock before leaving. */
    spin_unlock_irqrestore( &IrqOp_SpinLock, Flags );

    return( retvalue );

} /* end uviRequestUseVmeInt */


/********************************************************************************
*
*  FUNCTION:   uviReleaseVmeInt
*
*  PURPOSE:    To unlock the VME interrupt level/vector resource.
*              This will disable the interrupt if level in use count goes to 0.
*
*  PARAMETERS: vmelevel - vme interrupt level to use
*              vmevector - vme interrupt vector to use
*
*  RETURNS:    UVI_SUCCESS      : Success
*              UVI_BAD_PARAMETER_1
*              UVI_BAD_PARAMETER_2
*
*  GLOBALS:    IntTable, LvlInUseCnt
*
********************************************************************************/
UVI_INT32 uviReleaseVmeInt( UVI_INT32 vmelevel, UVI_INT32 vmevector )
{

    UVI_INT32 retvalue = UVI_SUCCESS;

    static char *ThisFunction = "uviReleaseVmeInt";

    /* Lock function while in use. */
    spin_lock_irqsave( &IrqOp_SpinLock, Flags );

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; level:%x vector: %x Entering ...\n",
                                     MODULE_NAME, ThisFunction, vmelevel, vmevector );

    /* check for valid level and vector */
    if( (vmelevel < 1) || (vmelevel > 7) )
    {
      retvalue = UVI_BAD_PARAMETER_1;
    }
    else if( (vmevector < 0) || (vmevector > 0xFF) )
    {
      retvalue = UVI_BAD_PARAMETER_2;
    }
    else
    {
      IntTable[vmelevel - 1][vmevector].IntrFlags = 0;
      if( LvlInUseCnt[vmelevel - 1] ) LvlInUseCnt[vmelevel - 1] -= 1;
      if( !(LvlInUseCnt[vmelevel - 1]) )
      {
        /* disable the VME int. if level in use count goes to 0 */ 
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= LINT_DIS_VIRQ( vmelevel ); 
      }
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; STAT: %i Exiting ...\n",
                                     MODULE_NAME, ThisFunction, retvalue );

    /* Make sure we release the lock before leaving. */
    spin_unlock_irqrestore( &IrqOp_SpinLock, Flags );

    return( retvalue );

} /* end uviReleaseVmeInt */



/********************************************************************************
*
*  FUNCTION:   uviEnableVmeInt
*
*  PURPOSE:    To enable a VME interrupt.
*
*  PARAMETERS: VmeIntData - Interrupt data structure
*
*  RETURNS:    UVI_SUCCESS      : Success
*              UVI_BAD_PARAMETER_1
*
*  GLOBALS:    UnivBaseAddr, int_wq[][]
*
********************************************************************************/
UVI_INT32 uviEnableVmeInt( volatile struct uviInterruptParameters * VmeIntData ) 
{
    UVI_UINT32 intstat;
    UVI_INT32 ret_status;
    unsigned long saveflags;

    static char *ThisFunction = "uviEnableVmeInt";

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; LVL: %i Entering ...\n",
                             MODULE_NAME, ThisFunction, VmeIntData->IntLevel ); 

    ret_status = UVI_SUCCESS;

    if( (VmeIntData->IntLevel < 1) || (VmeIntData->IntLevel > 7) )
      return( UVI_BAD_PARAMETER_1 );

    save_flags( saveflags );
    cli();
    *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_EN_VIRQ( VmeIntData->IntLevel );
    *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= LINT_EN_VIRQ( VmeIntData->IntLevel );
    IntTable[VmeIntData->IntLevel - 1][VmeIntData->IntVector].IntrFlags |= USER_READY_INT;
    interruptible_sleep_on( &int_wq[VmeIntData->IntLevel - 1][VmeIntData->IntVector] );
    restore_flags( saveflags );
    IntTable[VmeIntData->IntLevel - 1][VmeIntData->IntVector].IntrFlags &= ~USER_READY_INT;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; LVL: %i Awake NOW ...\n", 
         MODULE_NAME, ThisFunction, VmeIntData->IntLevel );

    /* setup to return level and vector in status information */
    VmeIntData->IntStatus = VmeIntData->IntLevel << 8;
    VmeIntData->IntStatus |= VmeIntData->IntVector;

    /* check for bus error during IACK cycle */
    intstat =  *(UVI_UINT32 *)( UnivBaseAddr + (V_STATID_BASE + ( (VmeIntData->IntLevel - 1) * 4) ) );
    intstat &= IACK_BUS_ERROR;
    if( intstat )
    {
       intstat = intstat << 8;
       VmeIntData->IntStatus |= intstat;
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; INTSTATUS: %i RETSTATUS: %i USECNT: %i Exiting ...\n", 
      MODULE_NAME, ThisFunction, VmeIntData->IntStatus, ret_status, LvlInUseCnt[VmeIntData->IntLevel - 1] );

    return( ret_status );

} /* end of 'uviEnableVmeInt */


/********************************************************************************
*
*  FUNCTION:   uviRequestUseMB
*
*  PURPOSE:    To lock the DMA resource if available.
*
*  PARAMETERS: mb - mailbox to lock
*
*  RETURNS:    UVI_SUCCESS      : Success
*              UVI_MB_BUSY
*              UVI_BAD_PARAMETER_1
*
*  GLOBALS:    MB0_lock, MB1_lock, MB2_lock, MB3_lock
*
********************************************************************************/
UVI_INT32 uviRequestUseMB( UVI_INT32 mb )
{

    UVI_INT32 retvalue = 0;

    static char *ThisFunction = "uviRequestUseMB";

    /* Lock function while in use. */
    spin_lock_irqsave( &IrqOp_SpinLock, Flags );

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; mb: %i Entering ...\n",
                                     MODULE_NAME, ThisFunction, mb );
    /* check for valid mb number */
    if( (mb < 0) || (mb > 3) )
    {
      retvalue = UVI_BAD_PARAMETER_1;
    }
    else
    {
      switch( mb )
      {
        case 0:
          /* check for busy */
          atomic_inc( &MB0_lock );
          if( atomic_read( &MB0_lock ) > 1 ) 
          { 
             atomic_dec( &MB0_lock );
             retvalue = UVI_MBOX_BUSY; 
          }
          else retvalue = UVI_SUCCESS;
        break;
        case 1:
          /* check for busy */
          atomic_inc( &MB1_lock );
          if( atomic_read( &MB1_lock ) > 1 ) 
          { 
             atomic_dec( &MB1_lock );
             retvalue = UVI_MBOX_BUSY; 
          }
          else retvalue = UVI_SUCCESS;
        break;
        case 2:
          /* check for busy */
          atomic_inc( &MB2_lock );
          if( atomic_read( &MB2_lock ) > 1 ) 
          { 
             atomic_dec( &MB2_lock );
             retvalue = UVI_MBOX_BUSY; 
          }
          else retvalue = UVI_SUCCESS;
        break;
        case 3:
          /* check for busy */
          atomic_inc( &MB3_lock );
          if( atomic_read( &MB3_lock ) > 1 ) 
          { 
             atomic_dec( &MB3_lock );
             retvalue = UVI_MBOX_BUSY; 
          }
          else retvalue = UVI_SUCCESS;
        break;
      }
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; STAT: %i Exiting ...\n",
                                     MODULE_NAME, ThisFunction, retvalue );

    /* Make sure we release the lock before leaving. */
    spin_unlock_irqrestore( &IrqOp_SpinLock, Flags );

    return( retvalue );

} /* end uviRequestUseMB */


/********************************************************************************
*
*  FUNCTION:   uviReleaseUseMB
*
*  PURPOSE:    To unlock the Mailbox resource.
*
*  PARAMETERS: mb - mailbox to release
*
*  RETURNS:    UVI_SUCCESS      : Success
*              UVI_BAD_PARAMETER_1
*
*  GLOBALS:    MB0_lock, MB1_lock, MB2_lock, MB3_lock
*
********************************************************************************/
UVI_INT32 uviReleaseUseMB( UVI_INT32 mb )
{

    static char *ThisFunction = "uviReleaseUseMB";

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; MB: %i Entering ...\n",
                                     MODULE_NAME, ThisFunction, mb );

    /* check for valid mb number */
    if( (mb < 0) || (mb > 3) )
    {
      return( UVI_BAD_PARAMETER_1 );
    }

    /* Lock function while in use. */
    spin_lock_irqsave( &IrqOp_SpinLock, Flags );

    switch( mb )
    {
      case 0:
        /* free mailbox lock */
        atomic_set( &MB0_lock, 0 );
      break;
      case 1:
        /* free mailbox lock */
        atomic_set( &MB1_lock, 0 );
      break;
      case 2:
        /* free mailbox lock */
        atomic_set( &MB2_lock, 0 );
      break;
      case 3:
        /* free mailbox lock */
        atomic_set( &MB3_lock, 0 );
      break;
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME, 
                                     ThisFunction );

    /* Make sure we release the lock before leaving. */
    spin_unlock_irqrestore( &IrqOp_SpinLock, Flags );

     return( UVI_SUCCESS );

} /* end uviReleaseUseMB */


/********************************************************************************
*
*  FUNCTION:   uviCancelMB
*
*  PURPOSE:    To cancel waiting for mailbox Mailbox.
*
*  PARAMETERS: mb - mailbox number
*
*  RETURNS:    UVI_SUCCESS      : Success
*              UVI_BAD_PARAMETER_1
*
*  GLOBALS:    UnivBaseAddr
*
********************************************************************************/
UVI_INT32 uviCancelMB( UVI_INT32 mb )
{

    static char *ThisFunction = "uviCancelMB";

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

    /* check for valid mb number */
    if( (mb < 0) || (mb > 3) )
    {
      return( UVI_BAD_PARAMETER_1 );
    }

    /* Lock function while in use. */
    spin_lock_irqsave( &IrqOp_SpinLock, Flags );

    switch( mb )
    {
      /* disable MB interrupt and wake sleeping thread */
      case 0:
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= ~LINT_MB0;
        wake_up_interruptible( &mb0_wq );
      break;
      case 1:
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= ~LINT_MB1;
        wake_up_interruptible( &mb1_wq );
      break;
      case 2:
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= ~LINT_MB2;
        wake_up_interruptible( &mb2_wq );
      break;
      case 3:
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= ~LINT_MB3;
        wake_up_interruptible( &mb3_wq );
      break;
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME, 
                                      ThisFunction );

    /* Make sure we release the lock before leaving. */
    spin_unlock_irqrestore( &IrqOp_SpinLock, Flags );

     return( UVI_SUCCESS );

} /* end uviCancelMB */


/********************************************************************************
*
*  FUNCTION:   uviEnableMB
*
*  PURPOSE:    To enable a mailbox interrupt.
*
*  PARAMETERS: mbnum - mailbox to enable
*              mbdat - pointer to save mailbox data to
*              userflag - user/kernel calling flag
*
*  RETURNS:    UVI_SUCCESS      : Success
*              UVI_BAD_PARAMETER_1
*              UVI_MBOX_CANCELED
*              UVI_MBOX_ERROR
*
*  GLOBALS:    UnivBaseAddr, mb0_wq, mb1_wq, mb2_wq, mb3_wq
*
********************************************************************************/
UVI_INT32 uviEnableMB( UVI_INT32 mbnum, UVI_UINT32 * mbdat, UVI_INT32 userflag ) 
{
    UVI_INT32 mb;
    UVI_INT32 ret_status = 0;
    unsigned long saveflags;

    static char *ThisFunction = "uviEnableMB";

    mb = mbnum;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; MB: %i FLG: %i Entering ...\n",
                             MODULE_NAME, ThisFunction, mb, userflag ); 

    if( (mb < 0) || (mb > 3) )
      return( UVI_BAD_PARAMETER_1 );

    if( userflag )
    {
      switch( mb )
      { /* R/W1C clear any old status and enable interrupt */
        case 0:
          save_flags( saveflags );
          cli();
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_MB0;
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= LINT_MB0;
          interruptible_sleep_on( &mb0_wq );
          restore_flags( saveflags );
          *mbdat = *(UVI_UINT32 *)( UnivBaseAddr + MBOX0 );
          ret_status = UVI_SUCCESS;
        break;
        case 1:
          save_flags( saveflags );
          cli();
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_MB1;
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= LINT_MB1;
          interruptible_sleep_on( &mb1_wq );
          restore_flags( saveflags );
          *mbdat = *(UVI_UINT32 *)( UnivBaseAddr + MBOX1 );
          ret_status = UVI_SUCCESS;
       break;
        case 2:
          save_flags( saveflags );
          cli();
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_MB2;
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= LINT_MB2;
          interruptible_sleep_on( &mb2_wq );
          restore_flags( saveflags );
          *mbdat = *(UVI_UINT32 *)( UnivBaseAddr + MBOX2 );
          ret_status = UVI_SUCCESS;
        break;
        case 3:
          save_flags( saveflags );
          cli();
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_MB3;
          *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= LINT_MB3;
          interruptible_sleep_on( &mb3_wq );
          restore_flags( saveflags );
          *mbdat = *(UVI_UINT32 *)( UnivBaseAddr + MBOX3 );
          ret_status = UVI_SUCCESS;
        break;
      }

      WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; MB: %i DAT: %X Awake NOW ...\n", 
           MODULE_NAME, ThisFunction, mb, *mbdat );
    }
    else
    { /* not user level */
      switch( mb )
      { /* R/W1C clear any old status and enable interrupt */
       case 0:
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_MB0;
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= LINT_MB0;
       break;
       case 1:
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_MB1;
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= LINT_MB1;
       break;
       case 2:
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_MB2;
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= LINT_MB2;
       break;
       case 3:
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = LINT_MB3;
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= LINT_MB3;
       break;
     }
      ret_status = UVI_SUCCESS;
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; RET: %i Exiting ...\n", 
                                     MODULE_NAME, ThisFunction, ret_status );

    return( ret_status );

} /* end of 'uviEnableMB' */


/********************************************************************************
*
*  We can handle DMA transfers on either a module to module basis or a user level
*  to module basis.
*
*  For module to module operations, the author of the requesting module will be
*  responsible for setting up and managing the DMA data buffer.  For a user to
*  module request, this module will allocate/manage a DMA buffer and carry out
*  the requested operation, moving data to/from the user level as needed.
*
*  This module/driver will enable DMA ops, initiate the data transfer, handle
*  errors, and disable DMA ops.
*
********************************************************************************/
  
/********************************************************************************
*
*  FUNCTION:   uviEnableDma
*
*  PURPOSE:    To allocate a large, contiguous buffer for dma use.  For now, we
*              require the user to reserve a large buffer by adding 'mem=xM' to
*              his LILO or LOADLIN file.  This reserves the requested amount of
*              physical ram, removing it from the total physical ram available.
*              Then the user can allocate all of that reserved physical ram or a
*              portion of it.  The user is responsible for managing this area of
*              memory.
*              To install the local universe interrupt handler for DMA related
*              interrupts.
*              To enable DMA operations in the universe interface.
*
*  PARAMETERS: DmaBuffBaseAddr : base address of physical DMA buffer
*              DmaBuffSize     : DMA buffer size
*              DmaBuffPtr      : pointer to the address of the DMA buffer, set
*                                for caller
* 
*  RETURNS:    UVI_SUCCESS  : Success
*              UVI_NULL_PTR : Null pointer obtained
*
*  GLOBALS:    UnivBaseAddr
*
********************************************************************************/
UVI_INT32 uviEnableDma( UVI_UINT32 DmaBuffBaseAddr,
                        UVI_UINT32 DmaBuffSize,
                        volatile UVI_UINT8 **DmaBuffPtr )
{

    static char *ThisFunction = "uviEnableDma";
  
    UVI_INT32 InstallIntrStatus;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );
 
/*
** Get a pointer to the DMA buffer area.
*/
    *DmaBuffPtr = (char *)ioremap_nocache( DmaBuffBaseAddr, DmaBuffSize );
    if ( DmaBuffPtr == NULL )
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; NULL returned from dma buffer ioremap, this"
                    " is a problem!\n", MODULE_NAME, ThisFunction );
        }
        return( UVI_NULL_PTR );
    }
    else
    {
        WHENDEBUG( UVI_DBEXTRA )
        {
            printk( KERN_INFO "%s:%s; Pointer value to be returned from dma buffer ioremap "
                    "is 0x%x.\n", MODULE_NAME, ThisFunction, (UVI_UINT32)*DmaBuffPtr );
        }
    }

/*
** Install the local universe interrupt handler for DMA operations.  The 'install'
** function checks if the handler is already installed.
*/
    InstallIntrStatus = uviInstallUniverseInterruptHandler();

/* Enable DMA interrupts. */
    LintEnDmaEn = 0x1;
    *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= ( LintEnDmaEn << 8 );
    VintEnDmaEn = 0x1;
    *(UVI_UINT32 *)( UnivBaseAddr + VINT_EN ) |= ( VintEnDmaEn << 8 );

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    return( UVI_SUCCESS );

} /* end of 'uviEnableDma' */

/********************************************************************************
*
*  FUNCTION:   uviRequestUseDma
*
*  PURPOSE:    To lock the DMA resource if available.
*
*  PARAMETERS: NONE
*
*  RETURNS:    UVI_SUCCESS      : Success
*              UVI_DMA_BUSY
*
*  GLOBALS:    dma_lock
*
********************************************************************************/
UVI_INT32 uviRequestUseDma( void )
{

    UVI_INT32 retvalue;

    static char *ThisFunction = "uviRequestUseDma";

    /* Lock function while in use. */
    spin_lock_irqsave( &IrqOp_SpinLock, Flags );

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );
    /* check for busy */
    atomic_inc( &dma_lock );
    if( atomic_read( &dma_lock ) > 1 ) 
    { 
       atomic_dec( &dma_lock );
       retvalue = UVI_DMA_BUSY; 
    }
    else
    {
      retvalue = UVI_SUCCESS;
      UserOwnDma = DMA_IN_USE;
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME, 
                                     ThisFunction );

    /* Make sure we release the lock before leaving. */
    spin_unlock_irqrestore( &IrqOp_SpinLock, Flags );

    return( retvalue );

} /* end uviRequestUseDma */


/********************************************************************************
*
*  FUNCTION:   uviReleaseUseDma
*
*  PURPOSE:    To unlock the DMA resource.
*
*  PARAMETERS: NONE
*
*  RETURNS:    UVI_SUCCESS      : Success
*            
*
*  GLOBALS:    dma_lock
*
********************************************************************************/
UVI_INT32 uviReleaseUseDma( void )
{

    static char *ThisFunction = "uviReleaseUseDma";

    /* Lock function while in use. */
    spin_lock_irqsave( &IrqOp_SpinLock, Flags );

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

    /* free dma lock */
    atomic_set( &dma_lock, 0 );

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME, 
                                     ThisFunction );

    /* Make sure we release the lock before leaving. */
    spin_unlock_irqrestore( &IrqOp_SpinLock, Flags );

     return( UVI_SUCCESS );

} /* end uviReleaseUseDma */


/********************************************************************************
*
*  FUNCTION:   uviAccessDmaBuffer
*
*  PURPOSE:    To read or write DMA data.
*
*  PARAMETERS: DmaBufAddr
*              VmeAddr
*              ByteCount
*              AddrSpace
*              MaxDataWidth
*              ToVmeBus
*              *DmaCompStat
*              userflag
*
*  RETURNS:    UVI_SUCCESS
*              Negative values for other errors
*
*  GLOBALS:    UnivBaseAddr
*
********************************************************************************/
UVI_INT32 uviAccessDmaBuffer( UVI_UINT32 DmaBufAddr,
                              UVI_UINT32 VmeAddr,
                              UVI_INT32  ByteCount,
                              UVI_INT32  AddrSpace,
                              UVI_UINT8  MaxDataWidth,
                              UVI_BOOL   ToVmeBus,
                              UVI_INT32  *DmaCompStat,
                              UVI_INT32  userflag )
{

    unsigned long saveflags;

    static char *ThisFunction = "uviAccessDmaBuffer";

    UVI_UINT8 Pgm,
              Super,
              Space,
              CycleType,
              DgcsGoVal,
              DgcsChainVal,
              DgcsVonVal,
              DgcsVoffVal,
              DgcsStopVal,
              DgcsHaltVal,
              DgcsDoneVal,
              DgcsLerrVal,
              DgcsVerrVal,
              DgcsPerrVal,
              DgcsIntrDoneVal,
              DgcsIntrLerrVal,
              DgcsIntrVerrVal,
              DgcsIntrPerrVal;
 
    UVI_UINT32 TransferCtrl,
               GenCtrl;

    UVI_INT32 ret_status;
  
    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

    /* check for already active */
    if( *(UVI_UINT32 *)( UnivBaseAddr + DGCS) & 0x8000 ) return( UVI_DMA_BUSY );

    /* Load the Vme address, Pci address, and transfer byte count registers. */
    *(UVI_UINT32 *)( UnivBaseAddr + DTBC ) = ByteCount;

    *(UVI_UINT32 *)( UnivBaseAddr + DLA ) = DmaBufAddr;

    *(UVI_UINT32 *)( UnivBaseAddr + DVA ) = VmeAddr;

    /* Set up and write the DMA transfer control register. */
    switch( (AddrSpace & 0x0F) )
    {
      case 0x08:
      case 0x09:
      case 0x0A:
      case 0x0B:
        Super = 0;
      break;

      case 0x0C:
      case 0x0D:
      case 0x0E:
      case 0x0F:
        Super = 1;
      break;

      default: return( UVI_INV_ADDRMOD );
    }

    switch( (AddrSpace & 0x30) )
    {
      case 0x00:
        Space = 2;
      break;

      case 0x30:
        Space = 1;
      break;

      default: Space = 0;
    }

    switch( (AddrSpace & 0x0F) )
    {
      case 0x0A:
      case 0x0E:
        Pgm = 1;
      break;

      default: Pgm = 0;
    }

    /* MBLT must have 64 bit width */
    switch( (AddrSpace & 0x0F) )
    {
      case 0x08:
      case 0x0C:
        MaxDataWidth = 3;
      break;
    }

    /* BLT & MBLT must have SC and BLK set */
    switch( (AddrSpace & 0x0F) )
    {
      case 0x08:
      case 0x0B:
      case 0x0C:
      case 0x0F:
        CycleType = 1;
      break;

      default: CycleType = 0;
    }

    TransferCtrl = ( ToVmeBus << 31 ) | ( MaxDataWidth << 22 ) | ( Space << 16 )
                     | ( Pgm << 14 ) | ( Super << 12 ) | ( CycleType << 8 );
    
    *(UVI_UINT32 *)( UnivBaseAddr + DCTL ) = TransferCtrl;

    /* Set up and write the DMA general control/status register. */
    DgcsGoVal = 0x1;
    DgcsChainVal = 0x0;
    DgcsVonVal = 0x0;
    DgcsVoffVal = 0x1;
    DgcsStopVal = 0x1;
    DgcsHaltVal = 0x1;
    DgcsDoneVal = 0x1;
    DgcsLerrVal = 0x1;
    DgcsVerrVal = 0x1;
    DgcsPerrVal = 0x1;
    DgcsIntrDoneVal = 0x1;
    DgcsIntrLerrVal = 0x1;
    DgcsIntrVerrVal = 0x1;
    DgcsIntrPerrVal = 0x1;
   
    GenCtrl = ( DgcsGoVal << 31 ) | ( DgcsChainVal << 27 ) | ( DgcsVonVal << 20 )
                    | ( DgcsVoffVal << 16 ) | ( DgcsStopVal << 14 ) | ( DgcsHaltVal << 13 )
                    | ( DgcsDoneVal << 11 ) | ( DgcsLerrVal << 10 ) | ( DgcsVerrVal << 9 )
                    | ( DgcsPerrVal << 8 ) | ( DgcsIntrDoneVal << 3 ) | ( DgcsIntrLerrVal << 2 )
                    | ( DgcsIntrVerrVal << 1 ) | DgcsIntrPerrVal;

    if( userflag )
    {
      save_flags( saveflags );
      cli();
      *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = 0x700;
      *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= 0x100;
      if( UserOwnDma & USER_OWNS_DMA ) UserOwnDma |= USER_READY_DMA;

      WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s Going to Sleep now\n",
                             MODULE_NAME, ThisFunction ); 

      *(UVI_UINT32 *)( UnivBaseAddr + DGCS ) = GenCtrl;
      interruptible_sleep_on( &dma_wq ); 
      restore_flags( saveflags );

      *DmaCompStat = *(UVI_UINT32 *)( UnivBaseAddr + DGCS );
      if( UserOwnDma & USER_OWNS_DMA ) UserOwnDma &= ~USER_READY_DMA;
    }
    else
    {
      *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) = 0x700;
      *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= 0x100;
      *DmaCompStat = *(UVI_UINT32 *)( UnivBaseAddr + DGCS );
    }

    ret_status = UVI_SUCCESS;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n",
                                     MODULE_NAME, ThisFunction );

    return( ret_status );

} /* end of 'uviAcessDmaBuffer' */


/********************************************************************************
*
*  FUNCTION:   uviDisableDma
*
*  PURPOSE:    To deallocate the dma buffer obtained by a call to uviAllocDmaBuffer.
*              The user must give us a valid buffer pointer.
*
*  PARAMETERS: DmaBuffPtr,
*
*  RETURNS:    UVI_SUCCESS      : Success
*              Negative values for other errors
*
*  GLOBALS:    UnivBaseAddr
*
********************************************************************************/
UVI_INT32 uviDisableDma( volatile UVI_UINT8 *DmaBuffPtr )
{

    static char *ThisFunction = "uviDisableDma";

    UVI_INT32 RemoveIntrStatus;
  
    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

    WHENDEBUG( UVI_DBEXTRA )
    {
        printk( KERN_INFO "%s:%s; Pointer to dma buffer to unmap is %x.\n", MODULE_NAME,
                ThisFunction, (UVI_UINT32)DmaBuffPtr );
    }

    if ( DmaBuffPtr != NULL )
    {
        iounmap( (unsigned char *)DmaBuffPtr );
    }
    else
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; A problem occured trying to unmap dma buffer.\n",
                    MODULE_NAME, ThisFunction );
        }
    }

/*
** Remove the local universe interrupt handler.  The 'remove' function checks
** if anyone else still needs this handler.
*/
    RemoveIntrStatus = uviRemoveUniverseInterruptHandler();

/* Disable DMA interrupts. */
    LintEnDmaEn = 0x0;
    *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) |= ( LintEnDmaEn << 8 );
    VintEnDmaEn = 0x0;
    *(UVI_UINT32 *)( UnivBaseAddr + VINT_EN ) |= ( VintEnDmaEn << 8 );

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    return( UVI_SUCCESS );

} /* end of 'uviDisableDma' */


/********************************************************************************
*
*  FUNCTION:   uviConfigSlaveWin
*
*  PURPOSE:    To Open a VME slave window.
*
*  PARAMETERS: struct * uviVmeSlaveWinParameters
*
*  RETURNS:    UVI_SUCCESS      : Success
*              Negative values for other errors
*
*  GLOBALS:    UnivBaseAddr
*
********************************************************************************/

UVI_INT32 uviConfigSlaveWin( volatile struct uviVmeSlaveWinParameters * SlaveWin )
{

    static UVI_UINT32 slaveCTL[8] =
    {
      0xF00,
      0xF14,
      0xF28,
      0xF3C,
      0xF90,
      0xFA4,
      0xFB8,
      0xFCC
    };

    static UVI_UINT32 slaveBS[8] =
    {
      0xF04,
      0xF18,
      0xF2C,
      0xF40,
      0xF94,
      0xFA8,
      0xFBC,
      0xFD0
    };

    static UVI_UINT32 slaveBD[8] =
    {
      0xF08,
      0xF1C,
      0xF30,
      0xF44,
      0xF98,
      0xFAC,
      0xFC0,
      0xFD4
    };

    static UVI_UINT32 slaveTO[8] =
    {
      0xF0C,
      0xF20,
      0xF34,
      0xF48,
      0xF9C,
      0xFB0,
      0xFC4,
      0xFD8
    };

    static char *ThisFunction = "uviConfigSlaveWin";

    UVI_UINT32 slavectl;
  
    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

    /* check for non NULL pointers */
    if ( SlaveWin == NULL )
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; NULL pointer\n", MODULE_NAME, ThisFunction );
        }
        return( UVI_NULL_PTR );
    }

    if ( !SlaveWin->Fdesc )
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; NULL pointer\n", MODULE_NAME, ThisFunction );
        }
        return( UVI_NULL_PTR );
    }

    /* verify valid window number */
    if ( (SlaveWin->WinNum < 0) || (SlaveWin->WinNum > 7) )
    {
      return( UVI_BAD_PARAMETER_8 );
    }

    /* verify if window is already configured that the file descripter to reconfigure matches */
    if ( SlvWindowBusy[SlaveWin->WinNum] )
    {
      if ( SlvWindowBusy[SlaveWin->WinNum] != SlaveWin->Fdesc ) return( UVI_BAD_PARAMETER_1 );
    }

    /* verify granularity for window selected */
    if ( (SlaveWin->WinNum == 0) || (SlaveWin->WinNum == 4) )
    {
      if ( SlaveWin->WinSize & 0xFFF ) return( UVI_BAD_PARAMETER_6 );
    }
    else
    {
      if ( SlaveWin->WinSize & 0xFFFF ) return( UVI_BAD_PARAMETER_6 );
    }

    /* set VME base address */
    *(UVI_UINT32 *)(UnivBaseAddr + slaveBS[SlaveWin->WinNum]) = SlaveWin->VmeBusAddr;

    /* set VME bound address */
    *(UVI_UINT32 *)(UnivBaseAddr + slaveBD[SlaveWin->WinNum]) = 
        (SlaveWin->VmeBusAddr + SlaveWin->WinSize);

    /* set TO (translation offset) for local bus */
    /* calculate TO (translation offset) physical dram address (TO = LOCAL - VMEBASE) */
    *(UVI_UINT32 *)(UnivBaseAddr + slaveTO[SlaveWin->WinNum]) = 
        (SLAVE_BUFF_BASE + SlaveWin->Offset) - SlaveWin->VmeBusAddr;

    /* combine VME access parameters and enable window */
    slavectl = SlaveWin->SuperUser | SlaveWin->PgmData | SlaveWin->AddSpace;
    slavectl |= VSI_IMAGE_EN;
    *(UVI_UINT32 *)(UnivBaseAddr + slaveCTL[SlaveWin->WinNum]) = slavectl;

    /* mark window as busy with file descripter */
    SlvWindowBusy[SlaveWin->WinNum] = SlaveWin->Fdesc;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    return( UVI_SUCCESS );

} /* end of 'uviConfigSlaveWin' */


/********************************************************************************
*
*  FUNCTION:   uviCloseSlaveWin
*
*  PURPOSE:    To Close a VME slave window.
*
*  PARAMETERS: struct * uviVmeSlaveCloseParams
*
*  RETURNS:    UVI_SUCCESS      : Success
*              Negative values for other errors
*
*  GLOBALS:    UnivBaseAddr, SlvWindowBusy[]
*
********************************************************************************/

UVI_INT32 uviCloseSlaveWin( volatile struct uviVmeSlaveCloseParams * SlaveWin )
{
    static UVI_UINT32 slaveCTL[8] =
    {
      0xF00,
      0xF14,
      0xF28,
      0xF3C,
      0xF90,
      0xFA4,
      0xFB8,
      0xFCC
    };
    static char *ThisFunction = "uviConfigSlaveWin";

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

    /* check for non NULL pointers */
    if ( SlaveWin == NULL )
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; NULL pointer\n", MODULE_NAME, ThisFunction );
        }
        return( UVI_NULL_PTR );
    }

    if ( !SlaveWin->Fdesc )
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; NULL pointer\n", MODULE_NAME, ThisFunction );
        }
        return( UVI_NULL_PTR );
    }

    /* verify valid window number */
    if ( (SlaveWin->WinNum < 0) || (SlaveWin->WinNum > 7) )
    {
      return( UVI_BAD_PARAMETER_2 );
    }

    /* verify that the file descripter matches */
    if ( SlvWindowBusy[SlaveWin->WinNum] )
    {
      if ( SlvWindowBusy[SlaveWin->WinNum] != SlaveWin->Fdesc ) return( UVI_BAD_PARAMETER_1 );
    }
    else
    { /* already closed */
      return( UVI_SUCCESS );
    }

    *(UVI_UINT32 *)(UnivBaseAddr + slaveCTL[SlaveWin->WinNum]) = 0;

    SlvWindowBusy[SlaveWin->WinNum] = 0;

    return( UVI_SUCCESS );

} /* end uviCloseSlaveWin */

