/*
=================================================================================
                            COPYRIGHT NOTICE

    Copyright (C) 2000 VMIC
    International Copyright Secured.  All Rights Reserved.

---------------------------------------------------------------------------------
    Filename: uviregions.c  Version: 1.1  Modification Date: 00/03/21 10:15:04
---------------------------------------------------------------------------------
    Description: Get PCI regions information so we can avoid mapping VME devices
                 into these areas.
---------------------------------------------------------------------------------

    Revision History:
    Date         Who  Ver   Reason For Change
    -----------  ---  ----  -----------------------------------------------------
    21-MAR-2000  AE   1.1   Created.

=================================================================================
*/
#ident "@(#)uviregions.c 1.1 00/06/27 VMIC"


#include "uvi_module.h"

/********************************************************************************
*
*  FUNCTION:   uviGetPciRegions
*
*  PURPOSE:    Find the PCI regions to see if we may have a universe slave image
*              memory conflict.
*
*  PARAMETERS: void
*
*  RETURNS:    UVI_SUCCESS      : Success
*              UVI_MEM_CONFLICT : Possible memory use conflict.
*
*  GLOBALS:    PciDevice
*
********************************************************************************/
UVI_INT32 uviGetPciRegions( void )
{

    static char *ThisFunction = "uviGetPciRegions";
    char *RegionType;

    static UVI_UINT32 BaseAddresses[] =
    {
      PCI_BASE_ADDRESS_0,
      PCI_BASE_ADDRESS_1,
      PCI_BASE_ADDRESS_2,
      PCI_BASE_ADDRESS_3,
      PCI_BASE_ADDRESS_4,
      PCI_BASE_ADDRESS_5,
      0
    };

    UVI_INT32 ReadConfigStat,
              RegionSize;

    UVI_UINT32 CurrBaseAddr = 0,
               Mask = 0,
               AddrIndex;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                    ThisFunction );

    PciDevice = pci_devices;
    do
    {
        WHENDEBUG( UVI_DBEXTRA )
        {
            printk( KERN_INFO "%s:%s; PCI device found. Vendor id is %x, device id is"
                    " %x, dev function is %x.\n", MODULE_NAME, ThisFunction,
                    PciDevice->vendor, PciDevice->device, PciDevice->devfn );
        }

        for ( AddrIndex = 0; BaseAddresses[AddrIndex]; AddrIndex++ )
        {
            ReadConfigStat = pci_read_config_dword( PciDevice, BaseAddresses[AddrIndex],
                                                    &CurrBaseAddr );
            cli();
            pci_write_config_dword( PciDevice, BaseAddresses[AddrIndex], ~0 );
            ReadConfigStat = pci_read_config_dword( PciDevice, BaseAddresses[AddrIndex],
                                                    &Mask );
            pci_write_config_dword( PciDevice, BaseAddresses[AddrIndex], CurrBaseAddr );
            sti();

            if ( Mask )
            {
                WHENDEBUG( UVI_DBEXTRA ) 
                {
                    printk( KERN_INFO "%s: Region %d: mask 0x%x, now at 0x%x.\n",
                            ThisFunction, AddrIndex, (UVI_UINT32)Mask,
                            (UVI_UINT32)CurrBaseAddr );
                }
            }

            if ( Mask & PCI_BASE_ADDRESS_SPACE )
            {
                RegionType = "I/O";
                Mask &= PCI_BASE_ADDRESS_IO_MASK;
            }
            else
            {
                RegionType = "memory";
                RegionSize = ( ~Mask ) + 1;
                Mask &= PCI_BASE_ADDRESS_MEM_MASK;
                if ( ( CurrBaseAddr >= 0x40000000 && CurrBaseAddr <= 0xB0000000 ) ||
                     ( ( ( CurrBaseAddr + RegionSize ) >= 0x40000000 ) && ( CurrBaseAddr
                       <= 0xB0000000 ) ) )
                {
                    WHENDEBUG( UVI_DBEXTRA ) /* change to err when this code is ready */
                    {
                        printk( KERN_ERR "%s:%s; Possible memory area conflict.\n",
                                MODULE_NAME, ThisFunction );
                    }
                    return( UVI_MEM_CONFLICT );
                }
            }
 
        } /* end 'for' */

        PciDevice = PciDevice->next;

    } while ( PciDevice != ( struct pci_dev * )NULL );

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                    ThisFunction );

    return( UVI_SUCCESS );

} /* end 'uviGetPciRegions' */
