/*
=================================================================================
                            COPYRIGHT NOTICE

    Copyright (C) 2000 VMIC
    International Copyright Secured.  All Rights Reserved.

---------------------------------------------------------------------------------
    Filename: uviwindows_init.c  Version: 1.1  Modification Date: 00/03/21 10:15:04
---------------------------------------------------------------------------------
    Description: Setup initial VME windows.
---------------------------------------------------------------------------------

    Revision History:
    Date         Who  Ver   Reason For Change
    -----------  ---  ----  -----------------------------------------------------
    21-MAR-2000  AE   1.1   Created.

=================================================================================
*/
#ident "@(#)uviwindows_init.c 1.1 00/06/27 VMIC"


#include "uvi_module.h"

/********************************************************************************
*
*  FUNCTION:   uviInitVmeWindows
*
*  PURPOSE:    To setup VME window space to be made available by window operations
*              function.
*
*  PARAMETERS: None
*
*  RETURNS:    UVI_SUCCESS      : Success
*              Negative values for other errors
*
*  GLOBALS:    UnivBaseAddr, MaxA16UDSpace, MaxA16SDSpace, MaxA24UDSpace, MaxA24SDSpace
*              MaxA32UDSpace, MaxA32SDSpace
*
********************************************************************************/
UVI_INT32 uviInitVmeWindows( void )
{

    static char *ThisFunction = "uviInitVmeWindows";

    UVI_UINT32 TestCtrlRegVal;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME,
                                     ThisFunction );

/*
** Disable all image 'windows' prior to setting them up to avoid any unpleasant
** effects.
*/
    *(UVI_UINT32 *)( UnivBaseAddr + SLSI ) &= SLSI_TRGT_IMAGE_DIS;

    if ( Univ1Found )
    {
        WHENDEBUG( UVI_DBINIT )
        {
            printk( KERN_INFO "%s:%s; Disabling universe 1 windows.\n", MODULE_NAME,
                    ThisFunction );
        }
        *(UVI_UINT32 *)( UnivBaseAddr + LSI0_CTL ) &= TRGT_IMAGE_DIS;
        *(UVI_UINT32 *)( UnivBaseAddr + LSI1_CTL ) &= TRGT_IMAGE_DIS;
        *(UVI_UINT32 *)( UnivBaseAddr + LSI2_CTL ) &= TRGT_IMAGE_DIS;
        *(UVI_UINT32 *)( UnivBaseAddr + LSI3_CTL ) &= TRGT_IMAGE_DIS;
    }
    else
    {
        WHENDEBUG( UVI_DBINIT )
        {
            printk( KERN_INFO "%s:%s; Disabling universe 2 windows.\n", MODULE_NAME,
                    ThisFunction );
        }
        *(UVI_UINT32 *)( UnivBaseAddr + LSI0_CTL ) &= TRGT_IMAGE_DIS;
        *(UVI_UINT32 *)( UnivBaseAddr + LSI1_CTL ) &= TRGT_IMAGE_DIS;
        *(UVI_UINT32 *)( UnivBaseAddr + LSI2_CTL ) &= TRGT_IMAGE_DIS;
        *(UVI_UINT32 *)( UnivBaseAddr + LSI3_CTL ) &= TRGT_IMAGE_DIS;
        *(UVI_UINT32 *)( UnivBaseAddr + LSI4_CTL ) &= TRGT_IMAGE_DIS;
        *(UVI_UINT32 *)( UnivBaseAddr + LSI5_CTL ) &= TRGT_IMAGE_DIS;
        *(UVI_UINT32 *)( UnivBaseAddr + LSI6_CTL ) &= TRGT_IMAGE_DIS;
        *(UVI_UINT32 *)( UnivBaseAddr + LSI7_CTL ) &= TRGT_IMAGE_DIS;
    }

/*
** Each general purpose windows translation offset register is set so that the
** offset onto the VME bus is 0.  A16 window needs will be handled by the special
** PCI slave image.
*/
    *(UVI_UINT32 *)( UnivBaseAddr + LSI0_BS ) = WINDOW0_A24UD_BASE;
    *(UVI_UINT32 *)( UnivBaseAddr + LSI0_BD ) = WINDOW0_A24UD_BOUND;
    *(UVI_UINT32 *)( UnivBaseAddr + LSI0_TO ) = VME_A24UD_BASE - WINDOW0_A24UD_BASE;

    *(UVI_UINT32 *)( UnivBaseAddr + LSI1_BS ) = WINDOW1_A24SD_BASE;
    *(UVI_UINT32 *)( UnivBaseAddr + LSI1_BD ) = WINDOW1_A24SD_BOUND;
    *(UVI_UINT32 *)( UnivBaseAddr + LSI1_TO ) = VME_A24SD_BASE - WINDOW1_A24SD_BASE;

    *(UVI_UINT32 *)( UnivBaseAddr + LSI2_BS ) = WINDOW2_A32UD_BASE;
    *(UVI_UINT32 *)( UnivBaseAddr + LSI2_BD ) = WINDOW2_A32UD_BOUND;
    *(UVI_UINT32 *)( UnivBaseAddr + LSI2_TO ) = VME_A32UD_BASE - WINDOW2_A32UD_BASE;

    *(UVI_UINT32 *)( UnivBaseAddr + LSI3_BS ) = WINDOW3_A32SD_BASE;
    *(UVI_UINT32 *)( UnivBaseAddr + LSI3_BD ) = WINDOW3_A32SD_BOUND;
    *(UVI_UINT32 *)( UnivBaseAddr + LSI3_TO ) = VME_A32SD_BASE - WINDOW3_A32SD_BASE;

    MaxA16UDSpace = 0x0000FFFF;
    MaxA16SDSpace = 0x0000FFFF;

    MaxA24UDSpace = *(UVI_UINT32 *)( UnivBaseAddr + LSI0_BD ) - 
                                             *(UVI_UINT32 *)( UnivBaseAddr + LSI0_BS );

    MaxA24SDSpace = *(UVI_UINT32 *)( UnivBaseAddr + LSI1_BD ) - 
                                             *(UVI_UINT32 *)( UnivBaseAddr + LSI1_BS );

    MaxA32UDSpace = *(UVI_UINT32 *)( UnivBaseAddr + LSI2_BD ) - 
                                             *(UVI_UINT32 *)( UnivBaseAddr + LSI2_BS );

    MaxA32SDSpace = *(UVI_UINT32 *)( UnivBaseAddr + LSI3_BD ) - 
                                             *(UVI_UINT32 *)( UnivBaseAddr + LSI3_BS );

/*
** Write the PCI image control registers.  We are setting up windows with a
** static access mode.  The user will not be allowed to change a windows access
** mode, but an access mode will have to be sent to allow us to determine if
** a 'supported mode' window is available.  This determination is made in the
** uvimod_ops file.
**
** Write the special image control register first.
*/
    *(UVI_UINT32 *)( UnivBaseAddr + SLSI ) = SLSI_TRGT_IMAGE_EN |
                                             SLSI0_TRGT_VDW_32 |
                                             SLSI1_TRGT_VDW_32 |
                                             SLSI2_TRGT_VDW_32 |
                                             SLSI3_TRGT_VDW_32 |
                                             SLSI0_TRGT_PGM_DATA |
                                             SLSI1_TRGT_PGM_DATA |
                                             SLSI2_TRGT_PGM_DATA |
                                             SLSI3_TRGT_PGM_DATA |
                                             SLSI0_TRGT_SUPER_NP |
                                             SLSI1_TRGT_SUPER_P |
                                             SLSI2_TRGT_SUPER_NP |
                                             SLSI3_TRGT_SUPER_P |
                                             SLSI_TRGT_BS |
                                             SLSI_TRGT_LAS_MEM;

/*
** Write each general image control register.  The way we're setting up windows
** allows us to get by with using two general purpose windows for A24 space and
** two windows of large size, for A32 space.  Since we're only using four general
** purpose PCI target image windows we don't need to check whether we have a type
** 1 universe or type 2.
*/
    *(UVI_UINT32 *)( UnivBaseAddr + LSI0_CTL ) = TRGT_IMAGE_EN |
//                                                 TRGT_PWEN |
                                                 TRGT_VDW_32 |
                                                 TRGT_VAS_A24 |
                                                 TRGT_PGM_DATA |
                                                 TRGT_SUPER_NP |
                                                 TRGT_LAS_MEM;

    *(UVI_UINT32 *)( UnivBaseAddr + LSI1_CTL ) = TRGT_IMAGE_EN |
//                                                 TRGT_PWEN |
                                                 TRGT_VDW_32 |
                                                 TRGT_VAS_A24 |
                                                 TRGT_PGM_DATA |
                                                 TRGT_SUPER_P |
                                                 TRGT_LAS_MEM;

    *(UVI_UINT32 *)( UnivBaseAddr + LSI2_CTL ) = TRGT_IMAGE_EN |
//                                                 TRGT_PWEN |
                                                 TRGT_VDW_32 |
                                                 TRGT_VAS_A32 |
                                                 TRGT_PGM_DATA |
                                                 TRGT_SUPER_NP |
                                                 TRGT_LAS_MEM;

    *(UVI_UINT32 *)( UnivBaseAddr + LSI3_CTL ) = TRGT_IMAGE_EN |
//                                                 TRGT_PWEN |
                                                 TRGT_VDW_32 |
                                                 TRGT_VAS_A32 |
                                                 TRGT_PGM_DATA |
                                                 TRGT_SUPER_P |
                                                 TRGT_LAS_MEM;

    WHENDEBUG( UVI_DBEXTRA )
    {
        TestCtrlRegVal = *(UVI_UINT32 *)( UnivBaseAddr + SLSI );
        printk( KERN_INFO "%s:%s; Special PCI control register val is 0x%x.\n",
                MODULE_NAME, ThisFunction, TestCtrlRegVal );

        TestCtrlRegVal = *(UVI_UINT32 *)( UnivBaseAddr + LSI0_CTL );
        printk( KERN_INFO "%s:%s; A24UD PCI control register val is 0x%x.\n",
                MODULE_NAME, ThisFunction, TestCtrlRegVal );

        TestCtrlRegVal = *(UVI_UINT32 *)( UnivBaseAddr + LSI1_CTL );
        printk( KERN_INFO "%s:%s; A24SD PCI control register val is 0x%x.\n",
                MODULE_NAME, ThisFunction, TestCtrlRegVal );

        TestCtrlRegVal = *(UVI_UINT32 *)( UnivBaseAddr + LSI2_CTL );
        printk( KERN_INFO "%s:%s; A32UD PCI control register val is 0x%x.\n",
                MODULE_NAME, ThisFunction, TestCtrlRegVal );

        TestCtrlRegVal = *(UVI_UINT32 *)( UnivBaseAddr + LSI3_CTL );
        printk( KERN_INFO "%s:%s; A32SD PCI control register val is 0x%x.\n",
                MODULE_NAME, ThisFunction, TestCtrlRegVal );
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    return( UVI_SUCCESS );

}


/******************************************************************************** 
* 
*  FUNCTION:   uviInitMailboxes 
* 
*  PURPOSE:    To setup VME Mailbox space
*               
* 
*  PARAMETERS: None 
* 
*  RETURNS:    UVI_SUCCESS      : Success 
*              Negative values for other errors 
* 
*  GLOBALS:    UnivBaseAddr 
*               
* 
********************************************************************************/ 
UVI_INT32 uviInitMailboxes( void ) 
{ 
 
    static char *ThisFunction = "uviInitMailboxes"; 
 
 
    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME, 
                                     ThisFunction ); 
 
/* 
** Disable all Mailboxes prior to setting them up to avoid any unpleasant 
** effects. 
*/
    *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= 0x00F0FFFF;
    *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) |= 0x000F0000;
    *(UVI_UINT32 *)( UnivBaseAddr + VRAI_CTL ) = 0;
    *(UVI_UINT32 *)( UnivBaseAddr + LINT_MAP2 ) &= 0x77770000;
    *(UVI_UINT32 *)( UnivBaseAddr + MBOX0 ) = 0;
    *(UVI_UINT32 *)( UnivBaseAddr + MBOX1 ) = 0;
    *(UVI_UINT32 *)( UnivBaseAddr + MBOX2 ) = 0;
    *(UVI_UINT32 *)( UnivBaseAddr + MBOX3 ) = 0;

    /* if MB_REG_EN true then set them up */
    if( MB_REG_EN )
    {
      *(UVI_UINT32 *)( UnivBaseAddr + VRAI_CTL ) = (MB_REG_EN | MB_REG_PD |
          MB_REG_NPS | MB_REG_SPACE);
      *(UVI_UINT32 *)( UnivBaseAddr + VRAI_BS ) = MB_REG_ADD;
    }


    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n",
                                     MODULE_NAME, ThisFunction );   

    return( UVI_SUCCESS ); 
 
} 

